@@ -46,13 +46,10 @@ import xml.dom.minidom
 import xml.parsers.expat
 from fs import open_fs
 
-# related third party imports
 from mslib.msui.mss_qt import QtGui, QtCore, QtWidgets, variant_to_string, variant_to_float
-
-# local application imports
 from mslib import utils, __version__
 from mslib import thermolib
-from mslib.utils import config_loader, find_location, save_settings_pickle, load_settings_pickle
+from mslib.utils import config_loader, find_location, save_settings_qsettings, load_settings_qsettings
 from mslib.msui.performance_settings import DEFAULT_PERFORMANCE
 from mslib.msui import MissionSupportSystemDefaultConfig as mss_default
 
@@ -169,15 +166,13 @@ class WaypointsTableModel(QtCore.QAbstractTableModel):
     def load_settings(self):
         """Load settings from the file self.settingsfile.
         """
-        self.performance_settings = load_settings_pickle(self.settings_tag, DEFAULT_PERFORMANCE)
+        self.performance_settings = load_settings_qsettings(self.settings_tag, DEFAULT_PERFORMANCE)
 
     def save_settings(self):
         """Save the current settings (map appearance) to the file
            self.settingsfile.
         """
-        # TODO: ConfigParser and a central configuration file might be the better solution than pickle.
-        # http://stackoverflow.com/questions/200599/whats-the-best-way-to-store-simple-user-settings-in-python
-        save_settings_pickle(self.settings_tag, self.performance_settings)
+        save_settings_qsettings(self.settings_tag, self.performance_settings)
 
     def flags(self, index):
         """Used to specify which table columns can be edited by the user;
@@ -287,8 +282,8 @@ class WaypointsTableModel(QtCore.QAbstractTableModel):
                     value = variant_to_float(value)
                 except TypeError as ex:
                     logging.error("unexpected error: %s %s %s %s", type(ex), ex, type(value), value)
-                except ValueError:
-                    pass
+                except ValueError as ex:
+                    logging.error("{}".format(ex))
                 else:
                     waypoint.lat = value
                     waypoint.location = u""
@@ -312,8 +307,8 @@ class WaypointsTableModel(QtCore.QAbstractTableModel):
                     value = variant_to_float(value)
                 except TypeError as ex:
                     logging.error("unexpected error: %s %s %s %s", type(ex), ex, type(value), value)
-                except ValueError:
-                    pass
+                except ValueError as ex:
+                    logging.error("{}".format(ex))
                 else:
                     waypoint.lon = value
                     waypoint.location = u""
@@ -332,8 +327,8 @@ class WaypointsTableModel(QtCore.QAbstractTableModel):
                     pressure = float(thermolib.flightlevel2pressure(flightlevel))
                 except TypeError as ex:
                     logging.error("unexpected error: %s %s %s %s", type(ex), ex, type(value), value)
-                except ValueError:
-                    pass
+                except ValueError as ex:
+                    logging.error("{}".format(ex))
                 else:
                     waypoint.flightlevel = flightlevel
                     waypoint.pressure = pressure
@@ -353,8 +348,8 @@ class WaypointsTableModel(QtCore.QAbstractTableModel):
                     pressure = float(thermolib.flightlevel2pressure(flightlevel))
                 except TypeError as ex:
                     logging.error("unexpected error: %s %s %s %s", type(ex), ex, type(value), value)
-                except ValueError:
-                    pass
+                except ValueError as ex:
+                    logging.error("{}".format(ex))
                 else:
                     waypoint.pressure = pressure
                     waypoint.flightlevel = flightlevel
@@ -662,7 +657,7 @@ class WaypointDelegate(QtWidgets.QItemDelegate):
            combobox, get the corresponding coordinates.
         """
         if index.column() == LOCATION:
-            loc = str(editor.currentText())
+            loc = editor.currentText()
             locations = config_loader(dataset='locations', default=mss_default.locations)
             if loc in locations:
                 lat, lon = locations[loc]
