@@ -35,17 +35,13 @@ from datetime import datetime
 import os
 import six
 import logging
-
-# related third party imports
 import numpy as np
 import matplotlib
 import matplotlib.pyplot as plt
 from fs import open_fs
 from fslib.fs_filepicker import getSaveFileNameAndFilter
-
+from matplotlib import cbook
 from matplotlib.backends.backend_qt5agg import NavigationToolbar2QT, FigureCanvasQTAgg
-
-# local application imports
 from mslib import thermolib
 from mslib.utils import config_loader, FatalUserError
 from mslib.msui import MissionSupportSystemDefaultConfig as mss_default
@@ -55,7 +51,7 @@ from mslib.msui import mpl_map
 from mslib.msui import trajectory_item_tree as titree
 from mslib.msui.icons import icons
 from mslib.msui.mss_qt import QtCore, QtWidgets
-
+from mslib.utils import convert_pressure_to_vertical_axis_measure
 
 PIL_IMAGE_ORIGIN = "upper"
 LAST_SAVE_DIRECTORY = config_loader(dataset="data_dir", default=mss_default.data_dir)
@@ -183,8 +179,7 @@ def save_figure(self, *args):
         save_figure_original(self, *args)
     elif picker_type == "fs":
         filetypes = self.canvas.get_supported_filetypes_grouped()
-        sorted_filetypes = list(six.iteritems(filetypes))
-        sorted_filetypes.sort()
+        sorted_filetypes = sorted(six.iteritems(filetypes))
         startpath = matplotlib.rcParams.get('savefig.directory', LAST_SAVE_DIRECTORY)
         startpath = os.path.expanduser(startpath)
         start = os.path.join(startpath, self.canvas.get_default_filename())
@@ -246,6 +241,10 @@ class NavigationToolbar(NavigationToolbar2QT):
         # set the pointer icon and button press funcs to the
         # appropriate callbacks
 
+        # testing path_points
+        # x = [[10.1, 10.1, datetime(2012, 7, 1, 10, 30)], [13.0, 13.1, datetime(2012, 7, 1, 10, 30)]]
+        # y = path_points(x)
+        # logging.debug(y)
         if self._active == 'INSERT_WP':
             self._active = None
         else:
@@ -271,7 +270,6 @@ class NavigationToolbar(NavigationToolbar2QT):
             self.canvas.widgetlock(self)
         else:
             self.canvas.widgetlock.release(self)
-
         self.set_message(self.mode)
         self._update_buttons_checked()
 
@@ -364,6 +362,41 @@ class NavigationToolbar(NavigationToolbar2QT):
     def press_wp(self, event):
         self.canvas.waypoints_interactor.button_press_callback(event)
 
+    def mouse_move(self, event):
+        if not self.sideview:
+            self._set_cursor(event)
+            if event.inaxes and event.inaxes.get_navigate():
+                try:
+                    lat, lon = self.canvas.waypoints_interactor.get_lat_lon(event)
+                except (ValueError, OverflowError) as ex:
+                    logging.error("{}".format(ex))
+                else:
+                    s = "lat={:6.2f}, lon={:7.2f}".format(lat, lon)
+                    artists = [a for a in event.inaxes._mouseover_set
+                               if a.contains(event)[0] and a.get_visible()]
+                    if artists:
+                        a = cbook._topmost_artist(artists)
+                        if a is not event.inaxes.patch:
+                            data = a.get_cursor_data(event)
+                            if data is not None:
+                                data_str = a.format_cursor_data(data)
+                                if data_str is not None:
+                                    s += " " + data_str
+                    if len(self.mode):
+                        s = self.mode + ", " + s
+                    self.set_message(s)
+            else:
+                self.set_message(self.mode)
+        else:
+            if not event.ydata or not event.xdata:
+                self.set_message(self.mode)
+            else:
+                (lat, lon), _ = self.canvas.waypoints_interactor.get_lat_lon(event)
+                y_value = convert_pressure_to_vertical_axis_measure(
+                    self.canvas.settings_dict["vertical_axis"], event.ydata)
+                self.set_message("{} lat={:6.2f} lon={:7.2f} altitude={:.2f}".format(
+                    self.mode, lat, lon, y_value))
+
     def _init_toolbar(self):
         self.basedir = os.path.join(matplotlib.rcParams['datapath'], 'images')
         for text, tooltip_text, image_file, callback in self.toolitems:
@@ -381,14 +414,11 @@ class NavigationToolbar(NavigationToolbar2QT):
                     a = self.addAction(self._icon("qt4_editor_options.png"),
                                        'Customize', self.edit_parameters)
                     a.setToolTip('Edit axis, curve and image parameters')
-        if self.sideview:
-            wp_tools = [('Mv WP', icons("32x32", "wp_move.png"), 'Move waypoints', 'move_wp')]
-        else:
-            wp_tools = [
-                ('Mv WP', icons("32x32", "wp_move.png"), 'Move waypoints', 'move_wp'),
-                ('Ins WP', icons("32x32", "wp_insert.png"), 'Insert waypoints', 'insert_wp'),
-                ('Del WP', icons("32x32", "wp_delete.png"), 'Delete waypoints', 'delete_wp'),
-            ]
+        wp_tools = [
+            ('Mv WP', icons("32x32", "wp_move.png"), 'Move waypoints', 'move_wp'),
+            ('Ins WP', icons("32x32", "wp_insert.png"), 'Insert waypoints', 'insert_wp'),
+            ('Del WP', icons("32x32", "wp_delete.png"), 'Delete waypoints', 'delete_wp'),
+        ]
         self.addSeparator()
         for text, img, tooltip_text, callback in wp_tools:
             a = self.addAction(QtGui.QIcon(img), text, getattr(self, callback))
@@ -467,7 +497,6 @@ class MplSideViewCanvas(MplCanvas):
     """
 
     def __init__(self, model=None, settings=None, numlabels=None):
-
         """
         Arguments:
         model -- WaypointsTableModel defining the vertical section.
@@ -528,6 +557,9 @@ class MplSideViewCanvas(MplCanvas):
             )
 
     def redraw_yaxis(self):
+        """ Redraws the y-axis on map after setting the values from sideview options dialog box"""
+
+        self.checknconvert()
         vaxis = self.settings_dict["vertical_axis"]
         if vaxis == "pressure":
             # Compute the position of major and minor ticks. Major ticks are labelled.
@@ -544,8 +576,18 @@ class MplSideViewCanvas(MplCanvas):
                 major_ticks = np.resize(major_ticks,
                                         len_major_ticks + len(major_ticks2) - 1)
                 major_ticks[len_major_ticks:] = major_ticks2[1:]
+                if self.p_top < 1000:
+                    major_ticks3 = np.arange(major_ticks[-1], self.p_top - 1, -label_distance // 100)
+                    len_major_ticks = len(major_ticks)
+                    major_ticks = np.resize(major_ticks,
+                                            len_major_ticks + len(major_ticks3) - 1)
+                    major_ticks[len_major_ticks:] = major_ticks3[1:]
+                if major_ticks[-1] != self.p_top:
+                    top = np.arange(self.p_top, 0, -100000)
+                    major_ticks = np.append(major_ticks, top)
 
-            labels = ["{:d}".format(int(l / 100.)) for l in major_ticks]
+            labels = ["{}".format(int(l / 100.))
+                      if (l / 100.) - int(l / 100.) == 0 else "{}".format(float(l / 100.)) for l in major_ticks]
 
             # .. the same for the minor ticks ..
             p_top_minor = max(label_distance, self.p_top)
@@ -560,10 +602,16 @@ class MplSideViewCanvas(MplCanvas):
                 minor_ticks = np.resize(minor_ticks,
                                         len_minor_ticks + len(minor_ticks2) - 1)
                 minor_ticks[len_minor_ticks:] = minor_ticks2[1:]
+                if self.p_top < 10000:
+                    minor_ticks3 = np.arange(1000, self.p_top - 1, -label_distance_minor // 100)
+                    len_minor_ticks = len(minor_ticks)
+                    minor_ticks = np.resize(minor_ticks,
+                                            len_minor_ticks + len(minor_ticks3) - 1)
+                    minor_ticks[len_minor_ticks:] = minor_ticks3[1:]
             self.ax.set_ylabel("pressure (hPa)")
         elif vaxis == "pressure altitude":
-            major_heights = np.arange(0, 30, 2)
-            minor_heights = np.arange(0, 30, 0.25)
+            major_heights = np.arange(0, thermolib.pressure2flightlevel(self.p_top) * 0.03048, 2)
+            minor_heights = np.arange(0, thermolib.pressure2flightlevel(self.p_top) * 0.03048, 0.5)
             major_fl = 10 * major_heights / 0.3048
             minor_fl = 10 * minor_heights / 0.3048
             major_ticks = thermolib.flightlevel2pressure_a(major_fl)
@@ -571,8 +619,8 @@ class MplSideViewCanvas(MplCanvas):
             labels = major_heights
             self.ax.set_ylabel("pressure altitude (km)")
         elif vaxis == "flight level":
-            major_fl = np.arange(0, 1000, 50)
-            minor_fl = np.arange(0, 1000, 10)
+            major_fl = np.arange(0, 1551, 50)
+            minor_fl = np.arange(0, 1551, 10)
             major_ticks = thermolib.flightlevel2pressure_a(major_fl)
             minor_ticks = thermolib.flightlevel2pressure_a(minor_fl)
             labels = major_fl
@@ -593,6 +641,8 @@ class MplSideViewCanvas(MplCanvas):
         Vertical cross section code (log-p axis etc.) taken from
         mss_batch_production/visualisation/mpl_vsec.py.
         """
+        self.checknconvert()
+
         ax = self.ax
         self.fig.subplots_adjust(left=0.08, right=0.96,
                                  top=0.9, bottom=0.14)
@@ -649,6 +699,7 @@ class MplSideViewCanvas(MplCanvas):
         """Set the vertical extent of the view to the specified pressure
            values (hPa) and redraw the plot.
         """
+        self.checknconvert()
         changed = False
         if self.p_bot != pbot * 100:
             self.p_bot = pbot * 100
@@ -668,6 +719,8 @@ class MplSideViewCanvas(MplCanvas):
     def get_vertical_extent(self):
         """Returns the bottom and top pressure (hPa) of the plot.
         """
+        self.checknconvert()
+
         return (self.p_bot // 100), (self.p_top // 100)
 
     def draw_flight_levels(self):
@@ -795,6 +848,17 @@ class MplSideViewCanvas(MplCanvas):
         self.draw()
         logging.debug("done.")
 
+    def checknconvert(self):
+        """ Checks for current units of axis and convert the upper and lower limit
+        to pa(pascals) for the internal computation by code """
+
+        if self.settings_dict["vertical_axis"] == "pressure altitude":
+            self.p_bot = thermolib.flightlevel2pressure(self.settings_dict["vertical_extent"][0] * 32.80)
+            self.p_top = thermolib.flightlevel2pressure(self.settings_dict["vertical_extent"][1] * 32.80)
+        elif self.settings_dict["vertical_axis"] == "flight level":
+            self.p_bot = thermolib.flightlevel2pressure(self.settings_dict["vertical_extent"][0])
+            self.p_top = thermolib.flightlevel2pressure(self.settings_dict["vertical_extent"][1])
+
 
 class MplSideViewWidget(MplNavBarWidget):
     """MplNavBarWidget using an MplSideViewCanvas as the Matplotlib
@@ -806,11 +870,11 @@ class MplSideViewWidget(MplNavBarWidget):
             sideview=True, parent=parent, canvas=MplSideViewCanvas())
         # Disable some elements of the Matplotlib navigation toolbar.
         # Available actions: Home, Back, Forward, Pan, Zoom, Subplots,
-        #                    Customize, Save
+        #                    Customize, Save, Insert Waypoint, Delete Waypoint
         actions = self.navbar.actions()
         for action in actions:
             if action.text() in ["Home", "Back", "Forward", "Pan", "Zoom",
-                                 "Subplots", "Customize", "Ins WP", "Del WP"]:
+                                 "Subplots", "Customize"]:
                 action.setEnabled(False)
 
 
@@ -944,20 +1008,27 @@ class MplTopViewCanvas(MplCanvas):
         return self.map.crs
 
     def getBBOX(self):
-        """Get the bounding box of the map (returns a 4-tuple
-           lllon, lllat, urlon, urlat).
         """
-        # Get the bounding box of the current map view
-        # (bbox = llcrnrlon, llcrnrlat, urcrnrlon, urcrnrlat).
+        Get the bounding box of the map
+        (returns a 4-tuple llx, lly, urx, ury) in degree or meters.
+        """
+
         axis = self.ax.axis()
 
-        if self.map.bbox_units == "latlon":
+        if self.map.bbox_units == "degree":
             # Convert the current axis corners to lat/lon coordinates.
             axis0, axis2 = self.map(axis[0], axis[2], inverse=True)
             axis1, axis3 = self.map(axis[1], axis[3], inverse=True)
-            axis = [axis0, axis1, axis2, axis3]
+            bbox = (axis0, axis2, axis1, axis3)
+
+        elif self.map.bbox_units.startswith("meter"):
+            center_x, center_y = self.map(
+                *(float(_x) for _x in self.map.bbox_units[6:-1].split(",")))
+            bbox = (axis[0] - center_x, axis[2] - center_y, axis[1] - center_x, axis[3] - center_y)
+
+        else:
+            bbox = axis[0], axis[2], axis[1], axis[3]
 
-        bbox = (axis[0], axis[2], axis[1], axis[3])
         return bbox
 
     def clear_figure(self):
@@ -1260,7 +1331,6 @@ class MplTimeSeriesViewCanvas(MplCanvas):
                                if variable.getVariableName() in self.subPlots]
 
             if mode in ["GXPROPERTY_CHANGE"]:
-                #
                 # If only a graphics property has been changed: Apply setp with
                 # the new properties to all variables of trajectory 'item'.
                 for variable in variablesToPlot:
@@ -1273,7 +1343,6 @@ class MplTimeSeriesViewCanvas(MplCanvas):
                         linewidth=item.getGxElementProperty("general", "linewidth"))
 
             elif mode in ["REDRAW", "MARKER_CHANGE"]:
-                #
                 # Loop over all variables to plot.
                 for variable in variablesToPlot:
                     #
@@ -1292,15 +1361,12 @@ class MplTimeSeriesViewCanvas(MplCanvas):
                     #
                     # Plot the variable data with the colour determines above, store
                     # the plot instance in the variable's gxElements.
-                    plotInstance = ax.plot(x, y,
-                                           color=item.getGxElementProperty("general", "colour"),
-                                           linestyle=item.getGxElementProperty("general",
-                                                                               "linestyle"),
-                                           linewidth=item.getGxElementProperty("general",
-                                                                               "linewidth"))
-                    variable.setGxElementProperty("timeseries",
-                                                  u"instance::{}".format(self.identifier),
-                                                  plotInstance)
+                    plotInstance = ax.plot(
+                        x, y, color=item.getGxElementProperty("general", "colour"),
+                        linestyle=item.getGxElementProperty("general", "linestyle"),
+                        linewidth=item.getGxElementProperty("general", "linewidth"))
+                    variable.setGxElementProperty(
+                        "timeseries", u"instance::{}".format(self.identifier), plotInstance)
                     #
                     # If we've just plotted the pressure variable flip the y-axis
                     # upside down.
@@ -1335,9 +1401,9 @@ class MplTimeSeriesViewCanvas(MplCanvas):
                     # The bottommost subplot gets the x-label: The name of the time
                     # variable.
                     if index == len(self.subPlots) - 1:
-                        ax.set_xlabel("time [hr since " +
-                                      earliestStartTime.strftime("%Y-%m-%d %H:%M UTC") +
-                                      "]")
+                        ax.set_xlabel(
+                            "time [hr since {}]".format(earliestStartTime.strftime("%Y-%m-%d %H:%M UTC")))
+
         # Update the figure canvas.
         self.fig.canvas.draw()
 
