@@ -40,7 +40,6 @@ if os.getenv("PROJ_LIB") is None or os.getenv("PROJ_LIB") == "PROJ_LIB":
     conda_dir = conda_file_dir.split('lib')[0]
     proj_lib = os.path.join(os.path.join(conda_dir, 'share'), 'proj')
     os.environ["PROJ_LIB"] = proj_lib
-    # if proj4 is installed we have also in the base environment epsg data
     if not os.path.exists(proj_lib):
         os.makedirs(proj_lib)
         epsg_file = os.path.join(proj_lib, 'epsg')
@@ -63,7 +62,6 @@ import shutil
 import sys
 import types
 import fs
-import warnings
 
 from mslib import __version__
 from mslib.msui.mss_qt import ui_mainwindow as ui
@@ -80,8 +78,6 @@ from mslib.utils import config_loader, setup_logging
 from mslib.msui import MissionSupportSystemDefaultConfig as mss_default
 from mslib.plugins.io.csv import load_from_csv, save_to_csv
 from mslib.msui.icons import icons, python_powered
-
-# related third party imports
 from mslib.msui.mss_qt import QtGui, QtCore, QtWidgets, get_open_filename, get_save_filename
 
 try:
@@ -195,7 +191,6 @@ class MSSMainWindow(QtWidgets.QMainWindow, ui.Ui_MSSMainWindow):
             ctypes.windll.shell32.SetCurrentProcessExplicitAppUserModelID(myappid)
         except (ImportError, AttributeError) as error:
             logging.debug("AttributeError, ImportError Exception %s", error)
-            pass
         # Reference to the flight track that is currently displayed in the
         # views.
         self.active_flight_track = None
@@ -707,13 +702,6 @@ class MSSMainWindow(QtWidgets.QMainWindow, ui.Ui_MSSMainWindow):
 
 
 def main():
-    pyversion = platform.python_version()
-    warnings.simplefilter("always")
-    if pyversion.startswith('2'):
-        warnings.warn(
-            'You are using Python {}, which will no longer be supported in mss 1.8.0'.format(pyversion),
-            DeprecationWarning
-        )
     try:
         prefix = os.environ["CONDA_DEFAULT_ENV"]
     except KeyError:
@@ -741,6 +729,8 @@ def main():
         print("Version:", __version__)
         sys.exit()
 
+    setup_logging(args)
+
     if args.menu:
         # Experimental feature to get mss into application menue
         if platform.system() == "Linux":
@@ -756,6 +746,8 @@ def main():
             dirname = os.path.dirname(application_destination)
             if not os.path.exists(dirname):
                 os.makedirs(dirname)
+            if prefix:
+                prefix = "({})".format(prefix)
             desktop = desktop.format(prefix,
                                      os.path.join(sys.prefix, "bin", "mss"),
                                      icon_destination)
@@ -774,8 +766,6 @@ def main():
         logging.info("menu entry removed")
         sys.exit()
 
-    setup_logging(args)
-
     logging.info(u"MSS Version: %s", __version__)
     logging.info(u"Python Version: %s", sys.version)
     logging.info(u"Platform: %s (%s)", platform.platform(), platform.architecture())
