@@ -30,19 +30,16 @@ from builtins import str
 
 import logging
 import functools
-from mslib.utils import config_loader, save_settings_pickle, load_settings_pickle
+from mslib.utils import config_loader, save_settings_qsettings, load_settings_qsettings
 from mslib.msui import MissionSupportSystemDefaultConfig as mss_default
-
-# related third party imports
 from mslib.msui.mss_qt import QtGui, QtWidgets
-
-# local application imports
+from mslib.msui.mss_qt import QtCore
 from mslib.msui.mss_qt import ui_sideview_window as ui
 from mslib.msui.mss_qt import ui_sideview_options as ui_opt
 from mslib.msui.viewwindows import MSSMplViewWindow
 from mslib.msui import wms_control as wms
 from mslib.msui.icons import icons
-
+from mslib import thermolib
 
 # Dock window indices.
 WMS = 0
@@ -59,6 +56,7 @@ class MSS_SV_OptionsDialog(QtWidgets.QDialog, ui_opt.Ui_SideViewOptionsDialog):
         parent -- Qt widget that is parent to this widget.
         settings_dict -- dictionary containing sideview options.
         """
+        _translate = QtCore.QCoreApplication.translate
         super(MSS_SV_OptionsDialog, self).__init__(parent)
         self.setupUi(self)
 
@@ -72,6 +70,7 @@ class MSS_SV_OptionsDialog(QtWidgets.QDialog, ui_opt.Ui_SideViewOptionsDialog):
                                  "colour_ft_vertices": (0, 0, 0, 0),
                                  "colour_ft_waypoints": (0, 0, 0, 0),
                                  "colour_ft_fill": (0, 0, 0, 0)}
+        suffixes = [' hpa', ' km', ' hft']
         if settings_dict is not None:
             default_settings_dict.update(settings_dict)
         settings_dict = default_settings_dict
@@ -89,6 +88,8 @@ class MSS_SV_OptionsDialog(QtWidgets.QDialog, ui_opt.Ui_SideViewOptionsDialog):
         for i in range(self.cbVerticalAxis.count()):
             if self.cbVerticalAxis.itemText(i) == settings_dict["vertical_axis"]:
                 self.cbVerticalAxis.setCurrentIndex(i)
+                self.sbPbot.setSuffix(_translate("SideViewOptionsDialog", suffixes[i]))
+                self.sbPtop.setSuffix(_translate("SideViewOptionsDialog", suffixes[i]))
 
         self.cbDrawFlightLevels.setChecked(settings_dict["draw_flightlevels"])
         self.cbDrawFlightTrack.setChecked(settings_dict["draw_flighttrack"])
@@ -105,6 +106,7 @@ class MSS_SV_OptionsDialog(QtWidgets.QDialog, ui_opt.Ui_SideViewOptionsDialog):
             button.setPalette(palette)
 
         # Connect colour button signals.
+        self.cbVerticalAxis.view().pressed.connect(self.verticalunitsclicked)
         self.btFillColour.clicked.connect(functools.partial(self.setColour, "ft_fill"))
         self.btWaypointsColour.clicked.connect(functools.partial(self.setColour, "ft_waypoints"))
         self.btVerticesColour.clicked.connect(functools.partial(self.setColour, "ft_vertices"))
@@ -156,7 +158,7 @@ class MSS_SV_OptionsDialog(QtWidgets.QDialog, ui_opt.Ui_SideViewOptionsDialog):
            values are corrected.
         """
         try:
-            flightlevel = int(float(str(item.text())))
+            flightlevel = int(item.text())
         except Exception as ex:
             logging.debug("Wildecard Exception %s - %s.", type(ex), ex)
             flightlevel = 0
@@ -164,20 +166,20 @@ class MSS_SV_OptionsDialog(QtWidgets.QDialog, ui_opt.Ui_SideViewOptionsDialog):
             flightlevel = 0
         if flightlevel > 999:
             flightlevel = 999
-        item.setText(str(int(flightlevel)))
+        item.setText(str(flightlevel))
         self.tableWidget.sortItems(0)
 
     def get_flight_levels(self):
         """Returns the flight level values contained in the table.
         """
-        return [int(str(self.tableWidget.item(row, 0).text()))
+        return [int(self.tableWidget.item(row, 0).text())
                 for row in range(self.tableWidget.rowCount())]
 
     def get_settings(self):
         """Return settings dictionary with values from the GUI elements.
         """
         settings_dict = {
-            "vertical_extent": (int(self.sbPbot.value()), int(self.sbPtop.value())),
+            "vertical_extent": (float(self.sbPbot.value()), float(self.sbPtop.value())),
             "vertical_axis": self.cbVerticalAxis.currentText(),
             "flightlevels": self.get_flight_levels(),
             "draw_flightlevels": self.cbDrawFlightLevels.isChecked(),
@@ -193,6 +195,38 @@ class MSS_SV_OptionsDialog(QtWidgets.QDialog, ui_opt.Ui_SideViewOptionsDialog):
         }
         return settings_dict
 
+    def verticalunitsclicked(self, index):
+        _translate = QtCore.QCoreApplication.translate
+        unit = self.cbVerticalAxis.model().itemFromIndex(index)
+        currentunit = self.cbVerticalAxis.currentText()
+        if unit.text() == "pressure":
+            self.sbPbot.setSuffix(_translate("SideViewOptionsDialog", " hpa"))
+            self.sbPtop.setSuffix(_translate("SideViewOptionsDialog", " hpa"))
+            if currentunit == "pressure altitude":
+                self.sbPbot.setValue(thermolib.flightlevel2pressure(self.sbPbot.value() * 32.80) / 100)
+                self.sbPtop.setValue(thermolib.flightlevel2pressure(self.sbPtop.value() * 32.80) / 100)
+            elif currentunit == "flight level":
+                self.sbPbot.setValue(thermolib.flightlevel2pressure(self.sbPbot.value()) / 100)
+                self.sbPtop.setValue(thermolib.flightlevel2pressure(self.sbPtop.value()) / 100)
+        elif unit.text() == "pressure altitude":
+            self.sbPbot.setSuffix(_translate("SideViewOptionsDialog", " km"))
+            self.sbPtop.setSuffix(_translate("SideViewOptionsDialog", " km"))
+            if currentunit == "pressure":
+                self.sbPbot.setValue(thermolib.pressure2flightlevel(self.sbPbot.value() * 100) * 0.03048)
+                self.sbPtop.setValue(thermolib.pressure2flightlevel(self.sbPtop.value() * 100) * 0.03048)
+            elif currentunit == "flight level":
+                self.sbPbot.setValue(self.sbPbot.value() * 0.03048)
+                self.sbPtop.setValue(self.sbPtop.value() * 0.03048)
+        elif unit.text() == "flight level":
+            self.sbPbot.setSuffix(_translate("SideViewOptionsDialog", " hft"))
+            self.sbPtop.setSuffix(_translate("SideViewOptionsDialog", " hft"))
+            if currentunit == "pressure":
+                self.sbPbot.setValue(thermolib.pressure2flightlevel(self.sbPbot.value() * 100))
+                self.sbPtop.setValue(thermolib.pressure2flightlevel(self.sbPtop.value() * 100))
+            elif currentunit == "pressure altitude":
+                self.sbPbot.setValue(self.sbPbot.value() * 32.80)
+                self.sbPtop.setValue(self.sbPtop.value() * 32.80)
+
 
 class MSSSideViewWindow(MSSMplViewWindow, ui.Ui_SideViewWindow):
     """PyQt window implementing a matplotlib canvas as an interactive
@@ -229,6 +263,9 @@ class MSSSideViewWindow(MSSMplViewWindow, ui.Ui_SideViewWindow):
     def __del__(self):
         del self.mpl.canvas.waypoints_interactor
 
+    def update_predefined_maps(self, extra):
+        pass
+
     def openTool(self, index):
         """Slot that handles requests to open tool windows.
         """
@@ -243,6 +280,7 @@ class MSSSideViewWindow(MSSMplViewWindow, ui.Ui_SideViewWindow):
                                                   view=self.mpl.canvas,
                                                   wms_cache=config_loader(dataset="wms_cache",
                                                                           default=mss_default.wms_cache))
+                self.mpl.canvas.waypoints_interactor.signal_get_vsec.connect(widget.call_get_vsec)
             else:
                 raise IndexError("invalid control index")
             # Create the actual dock widget containing <widget>.
@@ -271,13 +309,11 @@ class MSSSideViewWindow(MSSMplViewWindow, ui.Ui_SideViewWindow):
         """Save the current settings (vertical extent, displayed flightlevels
            etc.) to the file self.settingsfile.
         """
-        # TODO: ConfigParser and a central configuration file might be the better solution than pickle.
-        # http://stackoverflow.com/questions/200599/whats-the-best-way-to-store-simple-user-settings-in-python
         settings = self.getView().get_settings()
-        save_settings_pickle(self.settings_tag, settings)
+        save_settings_qsettings(self.settings_tag, settings)
 
     def load_settings(self):
         """Load settings from the file self.settingsfile.
         """
-        settings = load_settings_pickle(self.settings_tag)
+        settings = load_settings_qsettings(self.settings_tag)
         self.getView().set_settings(settings)
