@@ -33,14 +33,16 @@ from past.builtins import basestring
 from builtins import str
 
 import time
-from datetime import datetime, timedelta
+import isodate
+from datetime import datetime
 
 import io
 import hashlib
 import logging
+import mpl_toolkits.basemap as basemap
 import os
 import requests
-import re
+import traceback
 import urllib.parse
 import xml.etree.ElementTree as etree
 from mslib.utils import config_loader
@@ -64,7 +66,7 @@ WMS_URL_LIST = QtGui.QStandardItemModel()
 
 
 def add_wms_urls(combo_box, url_list):
-    combo_box_urls = [str(combo_box.itemText(_i)) for _i in range(combo_box.count())]
+    combo_box_urls = [combo_box.itemText(_i) for _i in range(combo_box.count())]
     for url in (_url for _url in url_list if _url not in combo_box_urls):
         combo_box.addItem(url)
 
@@ -158,8 +160,10 @@ class MSSWebMapService(mslib.ogcwms.WebMapService):
         # According to the WMS 1.1.1 standard, dimension names must be
         # prefixed with a "dim_", except for the in the standard predefined
         # dimensions "time" and "elevation".
-        time_name = time_name if time_name == "time" else "dim_" + time_name
-        init_time_name = "dim_" + init_time_name
+        if time is not None:
+            time_name = time_name if time_name == "time" else "dim_" + time_name
+        if init_time is not None:
+            init_time_name = "dim_" + init_time_name
 
         # If the parameters time and init_time are given as datetime objects,
         # create formatted strings with the given formatter. If they are
@@ -246,8 +250,8 @@ class MSS_WMS_AuthenticationDialog(QtWidgets.QDialog, ui_pw.Ui_WMSAuthentication
     def getAuthInfo(self):
         """Return the entered username and password.
         """
-        return (str(self.leUsername.text()),
-                str(self.lePassword.text()))
+        return (self.leUsername.text(),
+                self.lePassword.text())
 
 
 class WMSMapFetcher(QtCore.QObject):
@@ -386,15 +390,13 @@ class WMSControlWidget(QtWidgets.QWidget, ui.Ui_WMSDockWidget):
 
     prefetch = QtCore.pyqtSignal([list], name="prefetch")
     fetch = QtCore.pyqtSignal([list], name="fetch")
+    signal_disable_cbs = QtCore.Signal(name="disable_cbs")
+    signal_enable_cbs = QtCore.Signal(name="enable_cbs")
 
-    def __init__(self, parent=None, crs_filter=".*",
-                 default_WMS=None, wms_cache=None, view=None):
+    def __init__(self, parent=None, default_WMS=None, wms_cache=None, view=None):
         """
         Arguments:
         parent -- Qt widget that is parent to this widget.
-        crs_filter -- display only those layers in the 'Layers' combobox
-                      whose names match this regexp. Can be used to filter
-                      layers. Default is to display all layers.
         default_WMS -- list of strings that specify WMS URLs that will be
                        displayed in the URL combobox as default values.
         """
@@ -412,17 +414,13 @@ class WMSControlWidget(QtWidgets.QWidget, ui.Ui_WMSDockWidget):
         if default_WMS is not None:
             add_wms_urls(self.cbWMS_URL, default_WMS)
 
-        # Compile regular expression used in crsAllowed() to filter
-        # layers accordings to their CRS.
-        self.crs_filter = re.compile(crs_filter)
-
         # Initially allowed WMS parameters and date/time formats.
         self.allowed_init_times = []
         self.allowed_valid_times = []
-        self.init_time_format = "%Y-%m-%dT%H:%M:%SZ"
-        self.valid_time_format = "%Y-%m-%dT%H:%M:%SZ"
-        self.init_time_name = ""
-        self.valid_time_name = ""
+        self.init_time_format = None
+        self.valid_time_format = None
+        self.init_time_name = None
+        self.valid_time_name = None
 
         self.layerChangeInProgress = False
         self.save_level = None
@@ -471,11 +469,9 @@ class WMSControlWidget(QtWidgets.QWidget, ui.Ui_WMSDockWidget):
 
         # Initialise date/time fields with current day, 00 UTC.
         self.dteInitTime.setDateTime(QtCore.QDateTime(
-            datetime.utcnow().replace(hour=0, minute=0, second=0,
-                                      microsecond=0)))
+            datetime.utcnow().replace(hour=0, minute=0, second=0, microsecond=0)))
         self.dteValidTime.setDateTime(QtCore.QDateTime(
-            datetime.utcnow().replace(hour=0, minute=0, second=0,
-                                      microsecond=0)))
+            datetime.utcnow().replace(hour=0, minute=0, second=0, microsecond=0)))
 
         # Connect slots and signals.
         # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
@@ -607,7 +603,6 @@ class WMSControlWidget(QtWidgets.QWidget, ui.Ui_WMSDockWidget):
         return wms
 
     def wms_url_changed(self, text):
-        text = str(text)
         wms = WMS_SERVICE_CACHE.get(text)
         if wms is not None and wms != self.wms:
             self.activate_wms(wms)
@@ -632,6 +627,7 @@ class WMSControlWidget(QtWidgets.QWidget, ui.Ui_WMSDockWidget):
     @QtCore.pyqtSlot(Exception)
     def display_exception(self, ex):
         logging.error(u"ERROR: %s %s", type(ex), ex)
+        logging.debug(u"%s", traceback.format_exc())
         QtWidgets.QMessageBox.critical(
             self, self.tr("Web Map Service"), self.tr(u"ERROR:\n{}\n{}".format(type(ex), ex)))
 
@@ -646,6 +642,8 @@ class WMSControlWidget(QtWidgets.QWidget, ui.Ui_WMSDockWidget):
     def clear_map(self):
         logging.debug("clear figure")
         self.view.clear_figure()
+        logging.debug("enabling checkboxes in map-options if any")
+        self.signal_enable_cbs.emit()
 
     def get_capabilities(self):
         """Query the WMS server in the URL combobox for its capabilities. Fill
@@ -655,7 +653,7 @@ class WMSControlWidget(QtWidgets.QWidget, ui.Ui_WMSDockWidget):
         # Load new WMS. Only add those layers to the combobox that can provide
         # the CRS that match the filter of this module.
 
-        base_url = str(self.cbWMS_URL.currentText())
+        base_url = self.cbWMS_URL.currentText()
         try:
             request = requests.get(base_url)
         except (requests.exceptions.TooManyRedirects,
@@ -714,19 +712,17 @@ class WMSControlWidget(QtWidgets.QWidget, ui.Ui_WMSDockWidget):
 
         # Parse layer tree of the wms object and discover usable layers.
         stack = list(wms.contents.values())
-        filtered_layers = []
+        filtered_layers = set()
         while len(stack) > 0:
             layer = stack.pop()
-            if len(layer.layers) == 0:
-                if self.crs_allowed(layer):
-                    cb_string = u"{} | {}".format(layer.title, layer.name)
-                    if cb_string not in filtered_layers:
-                        filtered_layers.append(cb_string)
-            else:
+            if len(layer.layers) > 0:
                 stack.extend(layer.layers)
+            elif self.is_layer_aligned(layer):
+                cb_string = u"{} | {}".format(layer.title, layer.name)
+                filtered_layers.add(cb_string)
         logging.debug("discovered %i layers that can be used in this view",
                       len(filtered_layers))
-        filtered_layers.sort()
+        filtered_layers = sorted(filtered_layers)
         self.cbLayer.addItems(filtered_layers)
         self.cbLayer.setEnabled(self.cbLayer.count() > 1)
         self.wms = wms
@@ -768,6 +764,9 @@ class WMSControlWidget(QtWidgets.QWidget, ui.Ui_WMSDockWidget):
         if len(filtered_layers) > 0:
             self.btGetMap.setEnabled(True)
 
+        # logic to disable fill continents, fill oceans on connection to
+        self.signal_disable_cbs.emit()
+
     def view_capabilities(self):
         """Open a WMSCapabilitiesBrowser dialog showing the capabilities
            document.
@@ -781,18 +780,6 @@ class WMSControlWidget(QtWidgets.QWidget, ui.Ui_WMSDockWidget):
             wmsbrws.setAttribute(QtCore.Qt.WA_DeleteOnClose)
             wmsbrws.show()
 
-    def crs_allowed(self, layer):
-        """Check whether the CRS in which the layer can be provided are allowed
-           by the filter that was given to this module (in the constructor).
-        """
-        if hasattr(layer, "crsOptions") and layer.crsOptions is not None:
-            for crs in layer.crsOptions:
-                # If one of the CRS supported by the layer matches the filter
-                # return True.
-                if self.crs_filter.match(crs):
-                    return True
-        return False
-
     @staticmethod
     def interpret_timestring(timestring, return_format=False):
         """Tries to interpret a given time string.
@@ -815,7 +802,6 @@ class WMSControlWidget(QtWidgets.QWidget, ui.Ui_WMSDockWidget):
                     return d
             except ValueError as error:
                 logging.debug("ValueError Exception %s", error)
-                pass
         return None
 
     def get_layer_object(self, layername):
@@ -836,6 +822,40 @@ class WMSControlWidget(QtWidgets.QWidget, ui.Ui_WMSDockWidget):
                     stack.extend(layer.layers)
         return None
 
+    def parse_time_extent(self, values):
+        format, times = None, []
+        for time_item in [i.strip() for i in values]:
+            try:
+                list_desc = time_item.split("/")
+
+                if len(list_desc) == 3:
+                    if format is None:
+                        format = self.interpret_timestring(list_desc[0], return_format=True)
+
+                    time_val = datetime.strptime(list_desc[0], format)
+                    if "current" in list_desc[1]:
+                        end_time = datetime.utcnow()
+                    else:
+                        end_time = datetime.strptime(list_desc[1], format)
+                    delta = isodate.parse_duration(list_desc[2])
+
+                    while time_val <= end_time:
+                        times.append(time_val)
+                        time_val += delta
+
+                elif len(list_desc) == 1:
+                    if format is None:
+                        format = self.interpret_timestring(time_item, return_format=True)
+                    times.append(datetime.strptime(time_item, format))
+
+                else:
+                    raise ValueError("value has incorrect number of entries.")
+
+            except Exception as ex:
+                logging.debug(u"Wildecard Exception %s - %s.", type(ex), ex)
+                logging.error(u"Can't understand time string '%s'. Please check the implementation.", time_item)
+        return format, times
+
     def layer_changed(self, index):
         """Slot that updates the <cbStyle> and <teLayerAbstract> GUI elements
            when the user selects a new layer in <cbLayer>.
@@ -849,8 +869,7 @@ class WMSControlWidget(QtWidgets.QWidget, ui.Ui_WMSDockWidget):
         layerobj = self.get_layer_object(layer)
         styles = layerobj.styles
         self.cbStyle.clear()
-        self.cbStyle.addItems([u"{} | {}".format(s, styles[s]["title"])
-                               for s in styles])
+        self.cbStyle.addItems([u"{} | {}".format(s, styles[s]["title"]) for s in styles])
         self.cbStyle.setEnabled(self.cbStyle.count() > 1)
 
         abstract_text = layerobj.abstract if layerobj.abstract else ""
@@ -864,206 +883,104 @@ class WMSControlWidget(QtWidgets.QWidget, ui.Ui_WMSDockWidget):
         save_init_time = self.dteInitTime.dateTime()
         save_valid_time = self.dteValidTime.dateTime()
 
-        enable_elevation = False
-        enable_inittime = False
-        enable_validtime = False
         self.cbLevel.clear()
         self.cbInitTime.clear()
         self.cbValidTime.clear()
-        # ~~~~ A) Elevation. ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
+
+        # Gather dimensions and extents. Dimensions must be declared at one place only, extents may be
+        # overwritten by leafs.
+        dimensions, extents = {}, {}
+        self.allowed_crs = None
         lobj = layerobj
         while lobj is not None:
-            if "elevation" in lobj.dimensions and "elevation" in lobj.extents:
-                units = lobj.dimensions["elevation"]["units"]
-                elev_list = [u"{} ({})".format(e.strip(), units) for e in
-                             lobj.extents["elevation"]["values"]]
-                self.cbLevel.addItems(elev_list)
-                if self.save_level in elev_list:
-                    idx = elev_list.index(self.save_level)
-                    self.cbLevel.setCurrentIndex(idx)
-                enable_elevation = True
-                break
-
-            # One step up in the level hierarchy: If no dimension values
-            # were identified, check if this layer inherits some from its
-            # parent.
+            dimensions.update(lobj.dimensions)
+            for key in lobj.extents:
+                if key not in extents:
+                    extents[key] = lobj.extents[key]
+            if self.allowed_crs is None:
+                self.allowed_crs = getattr(lobj, "crsOptions", None)
             lobj = lobj.parent
 
+        if self.allowed_crs is not None and \
+                self.parent() is not None and \
+                self.parent().parent() is not None:
+            logging.debug("Layer offers '%s' projections.", self.allowed_crs)
+            extra = [_code for _code in self.allowed_crs if _code.startswith("EPSG")]
+            extra = [_code for _code in sorted(extra) if _code[5:] in basemap.epsg_dict]
+            logging.debug("Selected '%s' for Combobox.", extra)
+
+            self.parent().parent().update_predefined_maps(extra)
+
+        for key in [_x for _x in extents.keys() if _x not in dimensions]:
+            logging.error("extent '%s' not in dimensions!", key)
+            del extents[key]
+
+        # ~~~~ A) Elevation. ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
+        enable_elevation = False
+        if "elevation" in extents:
+            units = dimensions["elevation"]["units"]
+            elev_list = [u"{} ({})".format(e.strip(), units) for e in
+                         extents["elevation"]["values"]]
+            self.cbLevel.addItems(elev_list)
+            if self.save_level in elev_list:
+                idx = elev_list.index(self.save_level)
+                self.cbLevel.setCurrentIndex(idx)
+            enable_elevation = True
+
         # ~~~~ B) Initialisation time. ~~~~~~~~~~~~~~~~~~~~~~~~
-        self.allowed_init_times = []
-        self.init_time_format = None
+        try:
+            self.init_time_name = [x for x in ["init_time", "reference_time", "run"] if x in extents][0]
+            enable_init_time = True
+        except IndexError:
+            enable_init_time = False
 
-        lobj = layerobj
-        while lobj is not None:
-            if "init_time" in lobj.dimensions and "init_time" in lobj.extents:
-                # MSS web map service.
-                self.init_time_name = "init_time"
-                enable_inittime = True
-            elif "run" in lobj.dimensions and "run" in lobj.extents:
-                # IBL web map service.
-                self.init_time_name = "run"
-                enable_inittime = True
-            elif "reference_time" in lobj.dimensions and "reference_time" in lobj.extents:
-                # To support http://geoservices.knmi.nl/cgi-bin/HARM_N25.cg
-                self.init_time_name = "reference_time"
-                enable_inittime = True
-
-            # Initialisation tag was found: Try to determine the format of
-            # the date/time strings and read the provided extent values.
-            # Add the extent values to the init time combobox.
-            if enable_inittime:
-                self.init_time_format = self.interpret_timestring(
-                    lobj.extents[self.init_time_name]["values"][0].strip(),
-                    return_format=True)
-                if self.init_time_format is not None:
-                    # (otherwise self.init_time_format remains an emtpy list)
-                    self.allowed_init_times = [datetime.strptime(val.strip(), self.init_time_format)
-                                               for val in lobj.extents[self.init_time_name]["values"]]
-                self.cbInitTime.addItems(lobj.extents[self.init_time_name]["values"])
-                break
-
-            # One step up in the level hierarchy: If no dimension values
-            # were identified, check if this layer inherits some from its
-            # parent.
-            lobj = lobj.parent
+        # Both time dimension and time extent tags were found. Try to determine the
+        # format of the date/time strings.
+        if enable_init_time:
+            self.init_time_format, self.allowed_init_times = self.parse_time_extent(
+                extents[self.init_time_name]["values"])
+            self.cbInitTime.addItems(
+                [_time.strftime(self.init_time_format) for _time in self.allowed_init_times])
+            logging.debug(u"determined init time format: '%s'", self.valid_time_format)
+            if len(self.allowed_init_times) == 0:
+                msg = "cannot determine init time format."
+                logging.error(msg)
+                QtWidgets.QMessageBox.critical(
+                    self, self.tr("Web Map Service"), self.tr(u"ERROR: {}".format(msg)))
 
         # ~~~~ C) Valid/forecast time. ~~~~~~~~~~~~~~~~~~~~~~~~~
-        self.cbValidOn.setToolTip("")
-        font = self.cbValidOn.font()
-        font.setBold(False)
-        self.cbValidOn.setFont(font)
+        try:
+            self.valid_time_name = [x for x in ["time", "forecast"] if x in extents][0]
+            enable_valid_time = True
+        except IndexError:
+            enable_valid_time = False
+
+        # TODO Relic from the past: Is this really necessary or even correct??
         vtime_no_extent = False
-        self.allowed_valid_times = []
-        self.valid_time_format = None
+        if not enable_valid_time and "time" in dimensions:
+            self.valid_time_name = "time"
+            enable_valid_time = True
+            vtime_no_extent = True
+            self.allowed_valid_times = []
 
-        lobj = layerobj
-        while lobj is not None:
-            if "time" in lobj.dimensions and "time" in lobj.extents:
-                self.valid_time_name = "time"
-                enable_validtime = True
-            elif "time" in lobj.dimensions:
-                enable_validtime = True
-                self.valid_time_name = "time"
-                vtime_no_extent = True
-            elif "forecast" in lobj.dimensions and "forecast" in lobj.extents:
-                # IBL web map service.
-                self.valid_time_name = "forecast"
-                enable_validtime = True
-            elif "time" in lobj.extents:
-                # fix for capability document of https://neowms.sci.gsfc.nasa.gov/wms/wms
-                self.valid_time_name = "time"
-                enable_validtime = True
+        # Both time dimension and time extent tags were found. Try to determine the
+        # format of the date/time strings.
+        if enable_valid_time and not vtime_no_extent:
+            self.valid_time_format, self.allowed_valid_times = self.parse_time_extent(
+                extents[self.valid_time_name]["values"])
+            self.cbValidTime.addItems(
+                [_time.strftime(self.valid_time_format) for _time in self.allowed_valid_times])
 
-            # Both time dimension and time extent tags were found. Try to determine the
-            # format of the date/time strings.
-            if enable_validtime and not vtime_no_extent:
-                items = [i.strip() for i in lobj.extents[self.valid_time_name]["values"]]
-                if len(items) > 0:
-                    if items[0].find("/") > 0:
-                        # A "/" indicates a time range. Do NOT put any
-                        # values in the combo box and try to determine the time
-                        # format. Times are specified only by the date/time element
-                        # in this case.
-                        self.valid_time_format = self.interpret_timestring(items[0].split("/")[0],
-                                                                           return_format=True)
-                        font = self.cbValidOn.font()
-                        font.setBold(True)
-                        self.cbValidOn.setFont(font)
-                        self.cbValidOn.setToolTip(items[0])
-                        vtime_no_extent = True
-
-                    else:  # (no time range)
-                        # Determine time format and try to determine allowed times.
-                        self.valid_time_format = self.interpret_timestring(items[0],
-                                                                           return_format=True)
-
-                        # Loop over all time items provided by the server:
-                        self.allowed_valid_times = []
-                        for time_item in items:
-                            interpretation_successful = True
-
-                            if time_item.find("/") > 0:
-                                # This item describes a list of time
-                                # values. Expand the list.
-                                list_desc = time_item.split("/")
-                                # We need three entries: start time, end time,
-                                # time interval.
-                                if len(list_desc) != 3:
-                                    interpretation_successful = False
-
-                                try:
-                                    time_val = datetime.strptime(list_desc[0],
-                                                                 self.valid_time_format)
-                                    end_time = datetime.strptime(list_desc[1],
-                                                                 self.valid_time_format)
-                                    # Parse interval strings of format "PT3H" (= 3 hours)
-                                    # or "PT24H" (24 hours).
-                                    time_interval_hours = int(
-                                        re.match(r"PT(\d+)H", list_desc[2]).group(1))
-
-                                    # Register all time values of the list.
-                                    while time_val <= end_time:
-                                        self.allowed_valid_times.append(time_val)
-                                        self.cbValidTime.addItem(
-                                            time_val.strftime(self.valid_time_format))
-                                        time_val += timedelta(hours=time_interval_hours)
-                                except Exception as ex:
-                                    logging.debug("Wildecard Exception %s - %s.", type(ex), ex)
-                                    interpretation_successful = False
-
-                            else:
-                                # Single item, no list.
-                                try:
-                                    self.allowed_valid_times.append(
-                                        datetime.strptime(time_item,
-                                                          self.valid_time_format))
-                                    self.cbValidTime.addItem(time_item)
-                                except Exception as ex:
-                                    logging.debug("Wildecard Exception %s - %s.", type(ex), ex)
-                                    interpretation_successful = False
-
-                            if not interpretation_successful:
-                                logging.error(u"Can't understand time string '%s'."
-                                              u" Please check the implementation.", time_item)
-
-            # No time extent tag was found: Set allowed_valid_times to None
-            # (used by validTimeChanged()).
-            if vtime_no_extent:
-                self.allowed_valid_times = None
-
-            if enable_validtime:
-                break
-
-            # One step up in the level hierarchy: If no dimension values
-            # were identified, check if this layer inherits some from its
-            # parent.
-            lobj = lobj.parent
-
-        logging.debug(u"determined init time format: '%s'", self.init_time_format)
-        if enable_inittime and self.init_time_format is None:
-            msg = "cannot determine init time format."
-            logging.warning(u"WARNING: %s", msg)
-            if self.cbInitTime.count() == 0:
-                # If no values could be read from the extent tag notify
-                # the user.
+            logging.debug(u"determined valid time format: '%s'", self.valid_time_format)
+            if len(self.allowed_valid_times) == 0:
+                msg = "cannot determine valid time format."
+                logging.error(msg)
                 QtWidgets.QMessageBox.critical(
                     self, self.tr("Web Map Service"), self.tr(u"ERROR: {}".format(msg)))
-            self.init_time_format = ""
-
-        logging.debug(u"determined valid time format: '%s'", self.valid_time_format)
-        if enable_validtime and self.valid_time_format is None:
-            msg = "cannot determine valid time format."
-            logging.warning(u"WARNING: %s", msg)
-            if self.cbValidTime.count() == 0:
-                # If no values could be read from the extent tag notify
-                # the user.
-                QtWidgets.QMessageBox.critical(
-                    self, self.tr("Web Map Service"), self.tr(u"ERROR: {}".format(msg)))
-            self.valid_time_format = ""
 
         self.enable_level_elements(enable_elevation)
-        self.enable_valid_time_elements(enable_validtime)
-        self.enable_init_time_elements(enable_inittime)
+        self.enable_valid_time_elements(enable_valid_time)
+        self.enable_init_time_elements(enable_init_time)
 
         # Check whether dimension strings can be interpreted. If not, disable
         # the sync to the date/time elements.
@@ -1100,20 +1017,15 @@ class WMSControlWidget(QtWidgets.QWidget, ui.Ui_WMSDockWidget):
          c) 'XX days', where XX is an integer.
         """
         try:
-            minutes = int(timestep_string.split(" min")[0])
-            return minutes * 60
+            return 60 * int(timestep_string.split(" min")[0])
         except ValueError as error:
             logging.debug("ValueError Exception %s", error)
-            pass
         try:
-            hours = int(timestep_string.split(" hour")[0])
-            return hours * 3600
+            return 3600 * int(timestep_string.split(" hour")[0])
         except ValueError as error:
             logging.debug("ValueError Exception %s", error)
-            pass
         try:
-            days = int(timestep_string.split(" days")[0])
-            return days * 86400
+            return 86400 * int(timestep_string.split(" days")[0])
         except ValueError as error:
             logging.debug("ValueError Exception %s", error)
             raise ValueError(u"cannot convert '{}' to seconds: wrong format.".format(timestep_string))
@@ -1124,7 +1036,7 @@ class WMSControlWidget(QtWidgets.QWidget, ui.Ui_WMSDockWidget):
         # Get QDateTime object from QtDateTimeEdit field.
         d = self.dteInitTime.dateTime()
         # Add value from cbInitTime_step and set new date.
-        secs = self.secs_from_timestep(str(self.cbInitTime_step.currentText()))
+        secs = self.secs_from_timestep(self.cbInitTime_step.currentText())
         self.dteInitTime.setDateTime(d.addSecs(-1. * secs))
         self.auto_update()
 
@@ -1134,7 +1046,7 @@ class WMSControlWidget(QtWidgets.QWidget, ui.Ui_WMSDockWidget):
         # Get QDateTime object from QtDateTimeEdit field.
         d = self.dteInitTime.dateTime()
         # Add value from cbInitTime_step and set new date.
-        secs = self.secs_from_timestep(str(self.cbInitTime_step.currentText()))
+        secs = self.secs_from_timestep(self.cbInitTime_step.currentText())
         self.dteInitTime.setDateTime(d.addSecs(secs))
         self.auto_update()
 
@@ -1142,7 +1054,7 @@ class WMSControlWidget(QtWidgets.QWidget, ui.Ui_WMSDockWidget):
         # Get QDateTime object from QtDateTimeEdit field.
         d = self.dteValidTime.dateTime()
         # Add value from cbInitTime_step and set new date.
-        secs = self.secs_from_timestep(str(self.cbValidTime_step.currentText()))
+        secs = self.secs_from_timestep(self.cbValidTime_step.currentText())
         self.dteValidTime.setDateTime(d.addSecs(-1. * secs))
         self.auto_update()
 
@@ -1150,7 +1062,7 @@ class WMSControlWidget(QtWidgets.QWidget, ui.Ui_WMSDockWidget):
         # Get QDateTime object from QtDateTimeEdit field.
         d = self.dteValidTime.dateTime()
         # Add value from cbInitTime_step and set new date.
-        secs = self.secs_from_timestep(str(self.cbValidTime_step.currentText()))
+        secs = self.secs_from_timestep(self.cbValidTime_step.currentText())
         self.dteValidTime.setDateTime(d.addSecs(secs))
         self.auto_update()
 
@@ -1216,10 +1128,10 @@ class WMSControlWidget(QtWidgets.QWidget, ui.Ui_WMSDockWidget):
         'strike through' to indicate an invalid time.
         """
         font = self.dteInitTime.font()
-        inittime_available = dt.toPyDateTime() in self.allowed_init_times
-        font.setStrikeOut(not inittime_available)
+        init_time_available = dt.toPyDateTime() in self.allowed_init_times
+        font.setStrikeOut(not init_time_available)
         self.dteInitTime.setFont(font)
-        if inittime_available:
+        if init_time_available and self.init_time_format is not None:
             iso8601_time = dt.toPyDateTime().strftime(self.init_time_format)
             index = self.cbInitTime.findText(iso8601_time)
             self.cbInitTime.setCurrentIndex(index)
@@ -1227,25 +1139,24 @@ class WMSControlWidget(QtWidgets.QWidget, ui.Ui_WMSDockWidget):
     def check_valid_time(self, dt):
         """Same as check_init_time, but for valid time.
         """
-        font = self.dteValidTime.font()
-        if self.allowed_valid_times is not None:
-            validtime_available = dt.toPyDateTime() in self.allowed_valid_times
-        else:
-            validtime_available = True
-        font.setStrikeOut(not validtime_available)
-        self.dteValidTime.setFont(font)
-        if validtime_available:
+        if self.allowed_valid_times:
+            valid_time_available = dt.toPyDateTime() in self.allowed_valid_times
             iso8601_time = dt.toPyDateTime().strftime(self.valid_time_format)
             index = self.cbValidTime.findText(iso8601_time)
             # setCurrentIndex also sets the date/time edit via signal.
             self.cbValidTime.setCurrentIndex(index)
+        else:
+            valid_time_available = True
+        font = self.dteValidTime.font()
+        font.setStrikeOut(not valid_time_available)
+        self.dteValidTime.setFont(font)
 
     def init_time_changed(self):
         """Slot to be called when the current index of the init time
            combo box is changed. The method tries to sync to the
            init time date/time edit.
         """
-        init_time = str(self.cbInitTime.currentText())
+        init_time = self.cbInitTime.currentText()
         if init_time != "":
             init_time = self.interpret_timestring(init_time)
             if init_time is not None:
@@ -1256,7 +1167,7 @@ class WMSControlWidget(QtWidgets.QWidget, ui.Ui_WMSDockWidget):
     def valid_time_changed(self):
         """Same as initTimeChanged(), but for the valid time elements.
         """
-        valid_time = str(self.cbValidTime.currentText())
+        valid_time = self.cbValidTime.currentText()
         if valid_time != "":
             valid_time = self.interpret_timestring(valid_time)
             if valid_time is not None:
@@ -1363,7 +1274,7 @@ class WMSControlWidget(QtWidgets.QWidget, ui.Ui_WMSDockWidget):
             if self.dteInitTime.isEnabled():
                 return self.dteInitTime.dateTime().toPyDateTime()
             else:
-                itime_str = str(self.cbInitTime.currentText())
+                itime_str = self.cbInitTime.currentText()
                 return itime_str
         else:
             return None
@@ -1375,7 +1286,7 @@ class WMSControlWidget(QtWidgets.QWidget, ui.Ui_WMSDockWidget):
             if self.dteValidTime.isEnabled():
                 return self.dteValidTime.dateTime().toPyDateTime()
             else:
-                vtime_str = str(self.cbValidTime.currentText())
+                vtime_str = self.cbValidTime.currentText()
                 return vtime_str
         else:
             return None
@@ -1427,6 +1338,21 @@ class WMSControlWidget(QtWidgets.QWidget, ui.Ui_WMSDockWidget):
         level = self.get_level()
         transparent = self.cbTransparent.isChecked()
 
+        def normalize_crs(crs):
+            if any(crs.startswith(_prefix) for _prefix in ("MSS", "AUTO")):
+                return crs.split(",")[0]
+            return crs
+
+        if normalize_crs(crs) not in self.allowed_crs:
+            ret = QtWidgets.QMessageBox.warning(
+                self, self.tr("Web Map Service"),
+                self.tr("WARNING: Selected CRS '{}' not contained in allowed list of supported CRS for this WMS\n"
+                        "({})\n"
+                        "Continue ?".format(crs, self.allowed_crs)),
+                QtWidgets.QMessageBox.Yes | QtWidgets.QMessageBox.No, QtWidgets.QMessageBox.No)
+            if ret != QtWidgets.QMessageBox.Yes:
+                return
+
         # get...Time() will return None if the corresponding checkboxes are
         # disabled. <None> objects passed to wms.getmap will not be included
         # in the query URL that is send to the server.
@@ -1503,11 +1429,11 @@ class WMSControlWidget(QtWidgets.QWidget, ui.Ui_WMSDockWidget):
 
                     ci_time, ci_level = self.cbValidTime.currentIndex(), self.cbLevel.currentIndex()
                     prefetch_key_values = \
-                        [("time", str(self.cbValidTime.itemText(ci_p)))
+                        [("time", self.cbValidTime.itemText(ci_p))
                          for ci_p in list(range(ci_time + 1, ci_time + 1 + pre_tfwd)) +
                             list(range(ci_time - pre_tbck, ci_time))
                          if 0 <= ci_p < self.cbValidTime.count()] + \
-                        [("level", str(self.cbLevel.itemText(ci_p)).split(" (")[0])
+                        [("level", self.cbLevel.itemText(ci_p).split(" (")[0])
                          for ci_p in range(ci_level - pre_lbck, ci_level + 1 + pre_lfwd)
                          if ci_p != ci_level and 0 <= ci_p < self.cbLevel.count()]
 
@@ -1534,9 +1460,11 @@ class WMSControlWidget(QtWidgets.QWidget, ui.Ui_WMSDockWidget):
         if img is None:
             return
 
-        complete_level = str(self.cbLevel.currentText())
+        complete_level = self.cbLevel.currentText()
         complete_level = complete_level if complete_level != "" else None
         self.display_retrieved_image(img, legend_img, layer, style, init_time, valid_time, complete_level)
+        # this is for cases where 'remove' button is clicked, then 'retrieve' is clicked
+        self.signal_disable_cbs.emit()
 
     def get_map(self):
         """Prototypical stub for getMap() function. Needs to be reimplemented
@@ -1628,14 +1556,10 @@ class VSecWMSControlWidget(WMSControlWidget):
        handle (non-standard) vertical sections.
     """
 
-    def __init__(self, parent=None, crs_filter="VERT:.*",
-                 default_WMS=None, waypoints_model=None,
+    def __init__(self, parent=None, default_WMS=None, waypoints_model=None,
                  view=None, wms_cache=None):
-        super(VSecWMSControlWidget, self).__init__(parent=parent,
-                                                   crs_filter=crs_filter,
-                                                   default_WMS=default_WMS,
-                                                   wms_cache=wms_cache,
-                                                   view=view)
+        super(VSecWMSControlWidget, self).__init__(
+            parent=parent, default_WMS=default_WMS, wms_cache=wms_cache, view=view)
         self.waypoints_model = waypoints_model
         self.btGetMap.clicked.connect(self.get_vsec)
 
@@ -1645,6 +1569,11 @@ class VSecWMSControlWidget(WMSControlWidget):
         """
         self.waypoints_model = model
 
+    @QtCore.Slot()
+    def call_get_vsec(self):
+        if self.btGetMap.isEnabled() and self.cbAutoUpdate.isChecked() and not self.layerChangeInProgress:
+            self.btGetMap.click()
+
     def get_vsec(self):
         """Slot that retrieves the vertical section and passes the image
            to the view.
@@ -1682,6 +1611,9 @@ class VSecWMSControlWidget(WMSControlWidget):
                                 valid_time=valid_time,
                                 style=style_title)
 
+    def is_layer_aligned(self, layer):
+        crss = getattr(layer, "crsOptions", None)
+        return crss is not None and any(crs.startswith("VERT") for crs in crss)
 
 #
 # CLASS HSecWMSControlWidget
@@ -1693,13 +1625,9 @@ class HSecWMSControlWidget(WMSControlWidget):
        handle (standard) horizontal sections, i.e. maps.
     """
 
-    def __init__(self, parent=None, crs_filter="EPSG:.*",
-                 default_WMS=None, view=None, wms_cache=None):
-        super(HSecWMSControlWidget, self).__init__(parent=parent,
-                                                   crs_filter=crs_filter,
-                                                   default_WMS=default_WMS,
-                                                   wms_cache=wms_cache,
-                                                   view=view)
+    def __init__(self, parent=None, default_WMS=None, view=None, wms_cache=None):
+        super(HSecWMSControlWidget, self).__init__(
+            parent=parent, default_WMS=default_WMS, wms_cache=wms_cache, view=view)
         self.btGetMap.clicked.connect(self.get_map)
 
     def level_changed(self):
@@ -1743,3 +1671,7 @@ class HSecWMSControlWidget(WMSControlWidget):
         self.view.draw_image(img)
         self.view.draw_legend(legend_img)
         self.view.waypoints_interactor.update()
+
+    def is_layer_aligned(self, layer):
+        crss = getattr(layer, "crsOptions", None)
+        return crss is not None and not any(crs.startswith("VERT") for crs in crss)
