@@ -5,10 +5,8 @@
     mslib.mswms.mswms
     ~~~~~~~~~~~~~~~~~
 
-    The module can be run with the Python PASTE framework as a stand-alone
-    server (simply execute this file with Python).
-
-    This file is part of mss.
+    The module can be run with the Python Flask framework and can be run as
+    python mswms.py.
 
     :copyright: Copyright 2016 Reimar Bauer
     :copyright: Copyright 2016-2019 by the mss team, see AUTHORS.
@@ -33,8 +31,6 @@ from __future__ import absolute_import
 # Hack to fix missing PROJ4 env var in root environment
 import os
 import setuptools
-import warnings
-import platform
 
 
 if os.getenv("PROJ_LIB") is None or os.getenv("PROJ_LIB") == "PROJ_LIB":
@@ -42,7 +38,6 @@ if os.getenv("PROJ_LIB") is None or os.getenv("PROJ_LIB") == "PROJ_LIB":
     conda_dir = conda_file_dir.split('lib')[0]
     proj_lib = os.path.join(os.path.join(conda_dir, 'share'), 'proj')
     os.environ["PROJ_LIB"] = proj_lib
-    # if proj4 is installed we have also in the base environment epsg data
     if not os.path.exists(proj_lib):
         os.makedirs(proj_lib)
         epsg_file = os.path.join(proj_lib, 'epsg')
@@ -51,24 +46,17 @@ if os.getenv("PROJ_LIB") is None or os.getenv("PROJ_LIB") == "PROJ_LIB":
                 fid.write("# Placeholder for epsg data")
 
 
-import paste.httpserver
 import argparse
 import logging
 import sys
 
 from mslib import __version__
-from mslib.mswms.wms import mss_wms_settings, mss_wms_auth
+from mslib.mswms.wms import mss_wms_settings
+from mslib.mswms.wms import app as application
 from mslib.utils import setup_logging
 
 
 def main():
-    pyversion = platform.python_version()
-    warnings.simplefilter("always")
-    if pyversion.startswith('2'):
-        warnings.warn(
-            'You are using Python {}, which will no longer be supported in mss 1.8.0'.format(pyversion),
-            DeprecationWarning
-        )
     parser = argparse.ArgumentParser()
     parser.add_argument("-v", "--version", help="show version", action="store_true", default=False)
     parser.add_argument("--host", help="hostname",
@@ -90,25 +78,9 @@ def main():
 
     setup_logging(args)
 
-    from mslib.mswms.wms import application
-    if mss_wms_settings.__dict__.get('enable_basic_http_authentication', False):
-        logging.debug("Enabling basic HTTP authentication. Username and "
-                      "password required to access the service.")
-
-        from paste.auth.basic import AuthBasicHandler
-        import hashlib
-
-        realm = 'Mission Support Web Map Service'
-
-        def authfunc(environ, username, password):
-            for u, p in mss_wms_auth.allowed_users:
-                if (u == username) and (p == hashlib.md5(password.encode('utf-8')).hexdigest()):
-                    return True
-            return False
-
-        application = AuthBasicHandler(application, realm, authfunc)
     logging.info(u"Configuration File: '{}'".format(mss_wms_settings.__file__))
-    paste.httpserver.serve(application, host=args.host, port=args.port, use_threadpool=args.use_threadpool)
+
+    application.run(args.host, args.port)
 
 
 if __name__ == '__main__':
