@@ -44,21 +44,27 @@
 from future import standard_library
 standard_library.install_aliases()
 
-
-from past.builtins import basestring
-
 import os
 import logging
 from datetime import datetime
 import traceback
-import paste
-import paste.request
-import paste.util.multidict
 import urllib.parse
-import warnings
-import platform
 from chameleon import PageTemplateLoader
 
+
+from flask import Flask, request, make_response
+from flask_httpauth import HTTPBasicAuth
+from mslib.mswms.utils import conditional_decorator
+
+# Flask basic auth's documentation
+# https://flask-basicauth.readthedocs.io/en/latest/#flask.ext.basicauth.BasicAuth.check_credentials
+
+app = Flask(__name__)
+auth = HTTPBasicAuth()
+
+realm = 'Mission Support Web Map Service'
+app.config['realm'] = realm
+
 try:
     import mss_wms_settings
 except ImportError as ex:
@@ -96,8 +102,29 @@ except ImportError as ex:
                          ("add_new_user_here", "add_md5_digest_of_PASSWORD_here")]
         __file__ = None
 
+
+if mss_wms_settings.__dict__.get('enable_basic_http_authentication', False):
+    logging.debug("Enabling basic HTTP authentication. Username and "
+                  "password required to access the service.")
+    import hashlib
+
+    def authfunc(username, password):
+        for u, p in mss_wms_auth.allowed_users:
+            if (u == username) and (p == hashlib.md5(password.encode('utf-8')).hexdigest()):
+                return True
+        return False
+
+    @auth.verify_password
+    def verify_pw(username, password):
+        if request.authorization:
+            auth = request.authorization
+            username = auth.username
+            password = auth.password
+        return authfunc(username, password)
+
+
 from mslib.mswms import mss_plot_driver
-from mslib.utils import CaseInsensitiveMultiDict
+from mslib.utils import CaseInsensitiveMultiDict, get_projection_params
 
 # Logging the Standard Output, which will be added to the Apache Log Files
 logging.basicConfig(level=logging.DEBUG,
@@ -155,8 +182,7 @@ class WMSServer(object):
                 layer = layer_class(self.hsec_drivers[dataset])
             except KeyError:
                 continue
-            logging.debug("registering horizontal section layer '%s' with "
-                          "dataset '%s'", layer.name, dataset)
+            logging.debug(u"registering horizontal section layer '%s' with dataset '%s'", layer.name, dataset)
             # Check if the current dataset has already been registered. If
             # not, check whether a suitable driver is available.
             if dataset not in self.hsec_layer_registry:
@@ -189,15 +215,7 @@ class WMSServer(object):
                     raise ValueError(u"dataset '%s' not available", dataset)
             self.vsec_layer_registry[dataset][layer.name] = layer
 
-    def is_service_exception(self, var):
-        """Returns True if the given parameter is an XML that contains
-           a service exception.
-        """
-        if isinstance(var, basestring) and var.find(b"<ServiceException") > 1:
-            return True
-        return False
-
-    def service_exception(self, code=None, text=""):
+    def create_service_exception(self, code=None, text=""):
         """Create a service exception XML from the XML template defined above.
 
         Arguments:
@@ -207,10 +225,9 @@ class WMSServer(object):
 
         Returns an XML as string.
         """
-        logging.debug("creating service exception..")
+        logging.error(u"creating service exception code='%s' text='%s'.", code, text)
         template = templates['service_exception.pt']
-        # return_format = "text/xml"
-        return [template(code=code, text=text), "text/xml"]
+        return template(code=code, text=text).encode("utf-8"), "text/xml"
 
     def get_capabilities(self, server_url=None):
         # ToDo find a more elegant method to do the same
@@ -221,17 +238,17 @@ class WMSServer(object):
             data_access_dict[key].setup()
 
         template = templates['get_capabilities.pt']
-        logging.debug("server-url '%s'", server_url)
+        logging.debug(u"server-url '%s'", server_url)
 
         # Horizontal Layers
         hsec_layers = []
         for dataset in self.hsec_layer_registry:
             for layer in self.hsec_layer_registry[dataset].values():
                 if layer.uses_time_dimensions() and len(layer.get_init_times()) == 0:
-                    logging.error("layer %s/%s has no init times!", layer, dataset)
+                    logging.error(u"layer %s/%s has no init times!", layer, dataset)
                     continue
                 if layer.uses_time_dimensions() and len(layer.get_all_valid_times()) == 0:
-                    logging.error("layer %s/%s has no valid times!", layer, dataset)
+                    logging.error(u"layer %s/%s has no valid times!", layer, dataset)
                     continue
                 hsec_layers.append((dataset, layer))
 
@@ -240,37 +257,35 @@ class WMSServer(object):
         for dataset in self.vsec_layer_registry:
             for layer in self.vsec_layer_registry[dataset].values():
                 if layer.uses_time_dimensions() and len(layer.get_init_times()) == 0:
-                    logging.error("layer %s/%s has no init times!", layer, dataset)
+                    logging.error(u"layer %s/%s has no init times!", layer, dataset)
                     continue
                 if layer.uses_time_dimensions() and len(layer.get_all_valid_times()) == 0:
-                    logging.error("layer %s/%s has no valid times!", layer, dataset)
+                    logging.error(u"layer %s/%s has no valid times!", layer, dataset)
                     continue
                 vsec_layers.append((dataset, layer))
 
-        # return_format = 'text/xml'
+        settings = mss_wms_settings.__dict__
         return_data = template(hsec_layers=hsec_layers, vsec_layers=vsec_layers, server_url=server_url,
-                               service_name=mss_wms_settings.__dict__.get("service_name", "OGC:WMS"),
-                               service_title=mss_wms_settings.__dict__.get("service_title",
-                                                                           "Mission Support System Web Map Service"),
-                               service_abstract=mss_wms_settings.__dict__.get("service_abstract", ""),
-                               service_contact_person=mss_wms_settings.__dict__.get("service_contact_person", ""),
-                               service_contact_organisation=mss_wms_settings.__dict__.get(
-                                   "service_contact_organisation", ""),
-                               service_contact_position=mss_wms_settings.__dict__.get("service_contact_position", ""),
-                               service_email=mss_wms_settings.__dict__.get("service_email", ""),
-                               service_address_type=mss_wms_settings.__dict__.get("service_address_type", ""),
-                               service_address=mss_wms_settings.__dict__.get("service_address", ""),
-                               service_city=mss_wms_settings.__dict__.get("service_city", ""),
-                               service_state_or_province=mss_wms_settings.__dict__.get("service_state_or_province", ""),
-                               service_post_code=mss_wms_settings.__dict__.get("service_post_code", ""),
-                               service_country=mss_wms_settings.__dict__.get("service_country", ""),
-                               service_fees=mss_wms_settings.__dict__.get("service_fees", ""),
-                               service_access_constraints=mss_wms_settings.__dict__.get(
+                               service_name=settings.get("service_name", "OGC:WMS"),
+                               service_title=settings.get("service_title", "Mission Support System Web Map Service"),
+                               service_abstract=settings.get("service_abstract", ""),
+                               service_contact_person=settings.get("service_contact_person", ""),
+                               service_contact_organisation=settings.get("service_contact_organisation", ""),
+                               service_contact_position=settings.get("service_contact_position", ""),
+                               service_email=settings.get("service_email", ""),
+                               service_address_type=settings.get("service_address_type", ""),
+                               service_address=settings.get("service_address", ""),
+                               service_city=settings.get("service_city", ""),
+                               service_state_or_province=settings.get("service_state_or_province", ""),
+                               service_post_code=settings.get("service_post_code", ""),
+                               service_country=settings.get("service_country", ""),
+                               service_fees=settings.get("service_fees", ""),
+                               service_access_constraints=settings.get(
                                    "service_access_constraints",
                                    "This service is intended for research purposes only."))
-        return return_data
+        return return_data.encode("utf-8"), "text/xml"
 
-    def produce_plot(self, environ, mode):
+    def produce_plot(self, query, mode):
         """
         Handler for a GetMap and GetVSec requests. Produces a plot with
         the parameters specified in the URL.
@@ -281,16 +296,15 @@ class WMSServer(object):
         """
         logging.debug("GetMap/GetVSec request. Interpreting parameters..")
 
-        # 1) Get the query parameters from the URL.
+        # 1) Make query parameters Case Insenstitive
         # =========================================
-        query = CaseInsensitiveMultiDict(paste.request.parse_dict_querystring(environ))
-
+        query = CaseInsensitiveMultiDict(query)
         # 2) Evaluate query parameters:
         # =============================
 
         # Image size.
         figsize = float(query.get('WIDTH', 900)), float(query.get('HEIGHT', 600))
-        logging.debug("  requested image size = %sx%s", figsize[0], figsize[1])
+        logging.debug(u"  requested image size = %sx%s", figsize[0], figsize[1])
 
         # Requested layers.
         layers = [layer for layer in query.get('LAYERS', '').strip().split(',') if layer]
@@ -312,7 +326,7 @@ class WMSServer(object):
             try:
                 init_time = datetime.strptime(init_time, "%Y-%m-%dT%H:%M:%SZ")
             except ValueError:
-                return self.service_exception(
+                return self.create_service_exception(
                     code="InvalidDimensionValue",
                     text="DIM_INIT_TIME has wrong format (needs to be 2005-08-29T13:00:00Z)")
         logging.debug(u"  requested initialisation time = '%s'", init_time)
@@ -323,89 +337,76 @@ class WMSServer(object):
             try:
                 valid_time = datetime.strptime(valid_time, "%Y-%m-%dT%H:%M:%SZ")
             except ValueError:
-                return self.service_exception(
+                return self.create_service_exception(
                     code="InvalidDimensionValue",
                     text="TIME has wrong format (needs to be 2005-08-29T13:00:00Z)")
         logging.debug(u"  requested (valid) time = '%s'", valid_time)
 
         # Coordinate reference system.
-        crs = query.get('SRS', 'EPSG:4326')
-        # Allow to request vertical sections via GetMap, if the specified CRS is
-        # of type "VERT:??".
+        crs = query.get('SRS', 'EPSG:4326').lower()
+        # Allow to request vertical sections via GetMap, if the specified CRS is of type "VERT:??".
         msg = None
-        if crs.startswith('VERT:LOGP'):
-            mode = "GetVSec"
-        elif crs.startswith('EPSG:'):
+        if crs.startswith('vert:logp'):
+            mode = "getvsec"
+        else:
             try:
-                epsg = int(crs[5:])
+                get_projection_params(crs)
             except ValueError:
-                msg = u"The requested CRS '{}' is not supported.".format(crs)
-        else:
-            msg = u"The requested CRS '{}' is not supported.".format(crs)
-
-        if msg is not None:
-            logging.error(msg)
-            return self.service_exception(code="InvalidSRS", text=msg)
-        else:
-            logging.debug(u"  requested coordinate reference system = '%s'", crs)
+                return self.create_service_exception(
+                    code="InvalidSRS", text=u"The requested CRS '{}' is not supported.".format(crs))
+        logging.debug(u"  requested coordinate reference system = '%s'", crs)
 
         # Create a frameless figure (WMS) or one with title and legend
         # (MSS specific)? Default is WMS mode (frameless).
-        noframe = query.get('FRAME', 'OFF').upper() == 'OFF'
+        noframe = query.get('FRAME', 'off').lower() == 'off'
 
-        # Transparancy.
-        transparent = query.get('TRANSPARENT', 'FALSE').upper() == 'TRUE'
+        # Transparency.
+        transparent = query.get('TRANSPARENT', 'false').lower() == 'true'
         if transparent:
             logging.debug("  requested transparent image")
 
         # Return format (image/png, text/xml, etc.).
-        return_format = query.get('FORMAT', 'IMAGE/PNG').lower()
+        return_format = query.get('FORMAT', 'image/png').lower()
         logging.debug(u"  requested return format = '%s'", return_format)
         if return_format not in ["image/png", "text/xml"]:
-            msg = u"unsupported FORMAT: '{}'".format(return_format)
-            logging.error(msg)
-            return self.service_exception(code="InvalidFORMAT", text=msg)
+            return self.create_service_exception(
+                code="InvalidFORMAT",
+                text=u"unsupported FORMAT: '{}'".format(return_format))
 
         # 3) Check GetMap/GetVSec-specific parameters and produce
         #    the image with the corresponding section driver.
         # =======================================================
-
-        if mode == "GetMap":
+        if mode == "getmap":
             # Check requested layer.
-            if (dataset not in self.hsec_layer_registry) or \
-               (layer not in self.hsec_layer_registry[dataset]):
-                msg = u"Invalid LAYER '{}.{}' requested".format(dataset, layer)
-                logging.error(msg)
-                return self.service_exception(code="LayerNotDefined", text=msg)
+            if (dataset not in self.hsec_layer_registry) or (layer not in self.hsec_layer_registry[dataset]):
+                return self.create_service_exception(
+                    code="LayerNotDefined",
+                    text=u"Invalid LAYER '{}.{}' requested".format(dataset, layer))
 
             # Check if the layer requires time information and if they are given.
             if self.hsec_layer_registry[dataset][layer].uses_time_dimensions():
                 if init_time is None:
-                    msg = "INIT_TIME not specified (use the DIM_INIT_TIME keyword)"
-                    logging.error(msg)
-                    return self.service_exception(code="MissingDimensionValue", text=msg)
+                    return self.create_service_exception(
+                        code="MissingDimensionValue",
+                        text="INIT_TIME not specified (use the DIM_INIT_TIME keyword)")
                 if valid_time is None:
-                    msg = "TIME not specified"
-                    logging.error(msg)
-                    return self.service_exception(code="MissingDimensionValue", text=msg)
+                    return self.create_service_exception(code="MissingDimensionValue", text="TIME not specified")
 
             # Check if the requested coordinate system is supported.
-            if not self.hsec_layer_registry[dataset][layer].support_epsg_code(epsg):
-                msg = u"The requested CRS 'EPSG:{:d}' is not supported.".format(epsg)
-                logging.error(msg)
-                return self.service_exception(code="InvalidSRS", text=msg)
+            if not self.hsec_layer_registry[dataset][layer].support_epsg_code(crs):
+                return self.create_service_exception(
+                    code="InvalidSRS",
+                    text=u"The requested CRS '{}' is not supported.".format(crs))
 
             # Bounding box.
             try:
                 bbox = [float(v) for v in query.get('BBOX', '-180,-90,180,90').split(',')]
             except ValueError:
-                msg = u"Invalid BBOX: {}".format(query.get("BBOX"))
-                logging.error(msg)
-                return self.service_exception(text=msg)
+                return self.create_service_exception(text=u"Invalid BBOX: {}".format(query.get("BBOX")))
 
             # Vertical level, if applicable.
             level = query.get('ELEVATION')
-            level = float(level) if level else None
+            level = float(level) if level is not None else None
             layer_datatypes = self.hsec_layer_registry[dataset][layer].required_datatypes()
             if any(_x in layer_datatypes for _x in ["pl", "al", "ml", "tl", "pv"]) and level is None:
                 # Use the default value.
@@ -413,72 +414,56 @@ class WMSServer(object):
             elif ("sfc" in layer_datatypes) and \
                     all(_x not in layer_datatypes for _x in ["pl", "al", "ml", "tl", "pv"]) and \
                     level is not None:
-                msg = u"ELEVATION argument not applicable for layer " \
-                      u"'{}'. Please omit this argument.".format(layer)
-                logging.error(msg)
-                return self.service_exception(text=msg)
+                return self.create_service_exception(
+                    text=u"ELEVATION argument not applicable for layer '{}'. Please omit this argument.".format(layer))
 
             plot_driver = self.hsec_drivers[dataset]
             try:
-                plot_driver.set_plot_parameters(self.hsec_layer_registry[dataset][layer],
-                                                bbox=bbox,
-                                                level=level,
-                                                init_time=init_time,
-                                                valid_time=valid_time,
-                                                style=style,
-                                                figsize=figsize,
-                                                epsg=epsg,
-                                                noframe=noframe,
-                                                transparent=transparent,
+                plot_driver.set_plot_parameters(self.hsec_layer_registry[dataset][layer], bbox=bbox, level=level,
+                                                crs=crs, init_time=init_time, valid_time=valid_time, style=style,
+                                                figsize=figsize, noframe=noframe, transparent=transparent,
                                                 return_format=return_format)
                 image = plot_driver.plot()
             except (IOError, ValueError) as ex:
                 logging.error(u"ERROR: %s %s", type(ex), ex)
-                msg = u"The data corresponding to your request " \
-                      u"is not available. Please check the " \
-                      u"times and/or levels you have specified." \
-                      u"\n\nError message: '{}'".format(ex)
-                return self.service_exception(text=msg)
+                logging.debug(u"%s", traceback.format_exc())
+                msg = u"The data corresponding to your request is not available. Please check the " \
+                      u"times and/or levels you have specified.\n\n" \
+                      u"Error message: '{}'".format(ex)
+                return self.create_service_exception(text=msg)
 
-        elif mode == "GetVSec":
+        elif mode == "getvsec":
             # Vertical secton path.
             path = query.get("PATH")
-            if not path:
-                return self.service_exception(text="PATH not specified")
+            if path is None:
+                return self.create_service_exception(text="PATH not specified")
             try:
                 path = [float(v) for v in path.split(',')]
                 path = [[lat, lon] for lat, lon in zip(path[0::2], path[1::2])]
             except ValueError:
-                msg = u"Invalid PATH: {}".format(path)
-                logging.error(msg)
-                return self.service_exception(text=msg)
+                return self.create_service_exception(text=u"Invalid PATH: {}".format(path))
             logging.debug(u"VSEC PATH: %s", path)
 
             # Check requested layers.
-            if (dataset not in self.vsec_layer_registry) or \
-                    (layer not in self.vsec_layer_registry[dataset]):
-                msg = u"Invalid LAYER '{}.{}' requested".format(dataset, layer)
-                return self.service_exception(code="LayerNotDefined", text=msg)
+            if (dataset not in self.vsec_layer_registry) or (layer not in self.vsec_layer_registry[dataset]):
+                return self.create_service_exception(
+                    code="LayerNotDefined",
+                    text=u"Invalid LAYER '{}.{}' requested".format(dataset, layer))
 
             # Check if the layer requires time information and if they are given.
             if self.vsec_layer_registry[dataset][layer].uses_time_dimensions():
                 if init_time is None:
-                    msg = "INIT_TIME not specified (use the "\
-                          "DIM_INIT_TIME keyword)"
-                    logging.error(msg)
-                    return self.service_exception(code="MissingDimensionValue", text=msg)
+                    return self.create_service_exception(
+                        code="MissingDimensionValue",
+                        text="INIT_TIME not specified (use the DIM_INIT_TIME keyword)")
                 if valid_time is None:
-                    msg = "TIME not specified"
-                    logging.error(msg)
-                    return self.service_exception(code="MissingDimensionValue", text=msg)
+                    return self.create_service_exception(code="MissingDimensionValue", text="TIME not specified")
 
             # Bounding box (num interp. points, p_bot, num labels, p_top).
             try:
                 bbox = [float(v) for v in query.get("BBOX", "101,1050,10,180").split(",")]
             except ValueError:
-                msg = u"Invalid BBOX: {}".format(query.get("BBOX"))
-                logging.error(msg)
-                return self.service_exception(text=msg)
+                return self.create_service_exception(text=u"Invalid BBOX: {}".format(query.get("BBOX")))
 
             plot_driver = self.vsec_drivers[dataset]
             try:
@@ -498,70 +483,56 @@ class WMSServer(object):
                 image = plot_driver.plot()
             except (IOError, ValueError) as ex:
                 logging.error(u"ERROR: %s %s", type(ex), ex)
-                msg = u"The data corresponding to your request " \
-                      u"is not available. Please check the " \
-                      u"times and/or path you have specified." \
-                      u"\n\nError message: {}".format(ex)
-                return self.service_exception(text=msg)
+                msg = u"The data corresponding to your request is not available. Please check the " \
+                      u"times and/or path you have specified.\n\n" \
+                      u"Error message: {}".format(ex)
+                return self.create_service_exception(text=msg)
 
         # 4) Return the produced image.
         # =============================
         return image, return_format
 
 
-app = WMSServer()
+server = WMSServer()
 
 
-def application(environ, start_response):
-    pyversion = platform.python_version()
-    warnings.simplefilter("always")
-    if pyversion.startswith('2'):
-        warnings.warn(
-            'You are using Python {}, which will no longer be supported in mss 1.8.0'.format(pyversion),
-            DeprecationWarning
-        )
+@app.route('/')
+@conditional_decorator(auth.login_required, mss_wms_settings.__dict__.get('enable_basic_http_authentication', False))
+def application():
     try:
-        query = CaseInsensitiveMultiDict(paste.request.parse_dict_querystring(environ))
-        logging.debug(u"ENVIRON: %s", environ)
-        request = query.get('request')
-        output = ""
-        return_format = 'text/plain'
-        status = '200 OK'
+        # Request info
+        query = request.args
 
-        url = paste.request.construct_url(environ)
+        # Processing
+        request_type = query.get('request')
+        if request_type is None:  # request_type may *actually* be set to None
+            request_type = ''
+        request_type = request_type.lower()
+
+        url = request.url
         server_url = urllib.parse.urljoin(url, urllib.parse.urlparse(url).path)
 
-        if request is None:
-            status = '404 Not Found'
-        elif request.lower() == 'getcapabilities':
-            return_format = 'text/xml'
-            return_data = app.get_capabilities(server_url)
-            output = return_data.encode('utf-8')
-        elif request.lower() in ['getmap', 'getvsec']:
-            return_data, return_format = app.produce_plot(environ, request)
-            # ToDo refactor
-            if not app.is_service_exception(return_data):
-                return_format = return_format.lower()  # MAYBE TO BE DELETED
-                output = return_data
-            else:
-                status = '404 Not Found'
-                return_format = "text/xml"
-                output = return_data.encode('utf-8')
+        if request_type == "getcapabilities":
+            return_data, return_format = server.get_capabilities(server_url)
+        elif request_type in ['getmap', 'getvsec']:
+            return_data, return_format = server.produce_plot(query, request_type)
+        else:
+            raise RuntimeError(u"Request type '{}' is not valid.".format(request))
 
-        response_headers = [('Content-type', return_format), ('Content-Length', str(len(output)))]
-        start_response(status, response_headers)
+        res = make_response(return_data, 200)
+        response_headers = [('Content-type', return_format), ('Content-Length', str(len(return_data)))]
+        for response_header in response_headers:
+            res.headers[response_header[0]] = response_header[1]
 
-        return [output]
+        return res
 
     except Exception as ex:
-        status = '404 NOT FOUND'
-        error_message = u"{}: {}".format(type(ex), ex)
-        # ToDo add a config var to disable output, replace by standard text, "Internal Server error"
-        error_message = error_message + "\n" + traceback.format_exc()
-        logging.error(error_message)
+        error_message = u"{}: {}\n".format(type(ex), ex)
         error_message = error_message.encode("utf-8")
 
-        response_headers = [('Content-type', 'text/plain'),
-                            ('Content-Length', str(len(error_message)))]
-        start_response(status, response_headers)
-        return [error_message]
+        response_headers = [('Content-type', 'text/plain'), ('Content-Length', str(len(error_message)))]
+        res = make_response(error_message, 404)
+        for response_header in response_headers:
+            res.headers[response_header[0]] = response_header[1]
+
+        return res
