@@ -31,28 +31,29 @@ from __future__ import division
 from past.builtins import basestring
 
 import datetime
+from fs import open_fs, errors
 import json
 import logging
-import os
-import pickle
 import netCDF4 as nc
-
 import numpy as np
-import paste.util.multidict
+import os
 from scipy.interpolate import interp1d
 from scipy.ndimage import map_coordinates
-from mslib.msui import MissionSupportSystemDefaultConfig
-from fs import open_fs, errors
+import werkzeug.datastructures
+
 try:
     import mpl_toolkits.basemap.pyproj as pyproj
 except ImportError:
     import pyproj
 
-from mslib.msui import constants
+from mslib.msui import constants, MissionSupportSystemDefaultConfig
+from mslib.thermolib import pressure2flightlevel
+from PyQt5 import QtCore
 
 
 class FatalUserError(Exception):
-    pass
+    def __init__(self, error_string):
+        logging.debug("{}".format(error_string))
 
 
 def config_loader(config_file=None, dataset=None, default=None):
@@ -129,16 +130,15 @@ def find_location(lat, lon, tolerance=5):
     :return: None or lat/lon, name
     """
     locations = config_loader(dataset='locations', default=MissionSupportSystemDefaultConfig.locations)
-    distances = [(get_distance((lat, lon), (loc_lat, loc_lon)), loc)
-                 for loc, (loc_lat, loc_lon) in locations.items()]
-    distances.sort()
+    distances = sorted([(get_distance((lat, lon), (loc_lat, loc_lon)), loc)
+                        for loc, (loc_lat, loc_lon) in locations.items()])
     if len(distances) > 0 and distances[0][0] <= tolerance:
         return locations[distances[0][1]], distances[0][1]
     else:
         return None
 
 
-def save_settings_pickle(tag, settings):
+def save_settings_qsettings(tag, settings):
     """
     Saves a dictionary settings to disk.
 
@@ -148,16 +148,17 @@ def save_settings_pickle(tag, settings):
     """
     assert isinstance(tag, basestring)
     assert isinstance(settings, dict)
-    settingsfile = os.path.join(constants.MSS_CONFIG_PATH, u"mss.{}.cfg".format(tag))
-    logging.debug("storing settings for %s to %s", tag, settingsfile)
+    q_settings = QtCore.QSettings("mss", "mss-core")
+    file_path = q_settings.fileName()
+    logging.debug("storing settings for %s to %s", tag, file_path)
     try:
-        with open(settingsfile, "wb") as fileobj:
-            pickle.dump(settings, fileobj)
+        q_settings.setValue(tag, QtCore.QVariant(settings))
     except (OSError, IOError) as ex:
         logging.warning("Problems storing %s settings (%s: %s).", tag, type(ex), ex)
+    return settings
 
 
-def load_settings_pickle(tag, default_settings=None):
+def load_settings_qsettings(tag, default_settings=None):
     """
     Loads a dictionary of settings from disk. May supply a dictionary of default settings
     to return in case the settings file is not present or damaged. The default_settings one will
@@ -171,20 +172,15 @@ def load_settings_pickle(tag, default_settings=None):
     if default_settings is None:
         default_settings = {}
     assert isinstance(default_settings, dict)
-    settingsfile = os.path.join(constants.MSS_CONFIG_PATH, u"mss.{}.cfg".format(tag))
-    _dirname, _name = os.path.split(settingsfile)
-    _fs = open_fs(_dirname)
     settings = {}
-    if not _fs.exists(_name):
-        logging.debug("settings file '%s' for %s not available", settingsfile, tag)
-    else:
-        logging.debug("loading settings file '%s' for %s", settingsfile, tag)
-        try:
-            with _fs.open(_name, "rb") as fileobj:
-                settings = pickle.load(fileobj)
-        except (pickle.UnpicklingError, ValueError, KeyError, OSError, IOError, ImportError, TypeError) as ex:
-            logging.error("Problems reloading stored %s settings (%s: %s). Switching to default",
-                          tag, type(ex), ex)
+    q_settings = QtCore.QSettings("mss", "mss-core")
+    file_path = q_settings.fileName()
+    logging.debug(u"loading settings for %s from %s", tag, file_path)
+    try:
+        settings = q_settings.value(tag)
+    except Exception as ex:
+        logging.error("Problems reloading stored %s settings (%s: %s). Switching to default",
+                      tag, type(ex), ex)
     if isinstance(settings, dict):
         default_settings.update(settings)
     return default_settings
@@ -239,23 +235,129 @@ def rotate_point(point, angle, origin=(0, 0)):
 
 
 def convertHPAToKM(press):
-    return ((288.15 / 0.0065)) * (1. - ((press / 1013.25)) ** ((1. / 5.255))) / 1000.
+    return (288.15 / 0.0065) * (1. - (press / 1013.25) ** (1. / 5.255)) / 1000.
 
 
-def get_projection_params(epsg):
-    if epsg.startswith("EPSG:"):
-        epsg = epsg[5:]
-    proj_params = None
-    if epsg == "4326":
-        proj_params = {"basemap": {"projection": "cyl"}, "bbox": "latlon"}
-    elif epsg == "9810":
-        proj_params = {"basemap": {"projection": "stere", "lat_0": 90.0, "lon_0": 0.0}, "bbox": "metres"}
-    elif epsg.startswith("777") and len(epsg) == 8:
-        lat_0, lon_0 = int(epsg[3:5]), int(epsg[5:])
-        proj_params = {"basemap": {"projection": "stere", "lat_0": lat_0, "lon_0": lon_0}, "bbox": "latlon"}
-    elif epsg.startswith("778") and len(epsg) == 8:
-        lat_0, lon_0 = int(epsg[3:5]), int(epsg[5:])
-        proj_params = {"basemap": {"projection": "stere", "lat_0": -lat_0, "lon_0": lon_0}, "bbox": "latlon"}
+def get_projection_params(proj):
+    proj = proj.lower()
+    if proj.startswith("crs:"):
+        raise ValueError("CRS not supported")
+
+        projid = proj[4:]
+        if projid == "84":
+            proj_params = {
+                "basemap": {"projection": "cyl"},
+                "bbox": "degree"}
+        else:
+            raise ValueError("unsupported CRS code: '%s'", proj)
+
+    elif proj.startswith("auto:"):
+        raise ValueError("AUTO not supported")
+
+        projid, unitsid, lon0, lat0 = proj[5:].split(",")
+        if projid == "42001":
+            proj_params = {
+                "basemap": {"projection": "tmerc", "lon_0": lon0, "lat_0": lat0},
+                "bbox": "meter({},{})".format(lon0, lat0)}
+        elif projid == "42002":
+            proj_params = {
+                "basemap": {"projection": "tmerc", "lon_0": lon0, "lat_0": lat0},
+                "bbox": "meter({},{})".format(lon0, lat0)}
+        elif projid == "42003":
+            proj_params = {
+                "basemap": {"projection": "ortho", "lon_0": lon0, "lat_0": lat0},
+                "bbox": "meter({},{})".format(lon0, lat0)}
+        else:
+            raise ValueError("unspecified AUTO code: '%s'", proj)
+
+    elif proj.startswith("auto2:"):
+        raise ValueError("AUTO2 not supported")
+
+        projid, factor, lon0, lat0 = proj[6:].split(",")
+        if projid == "42001":
+            proj_params = {
+                "basemap": {"projection": "tmerc", "lon_0": lon0, "lat_0": lat0},
+                "bbox": "meter({},{})".format(lon0, lat0)}
+        elif projid == "42002":
+            proj_params = {
+                "basemap": {"projection": "tmerc", "lon_0": lon0, "lat_0": lat0},
+                "bbox": "meter({},{})".format(lon0, lat0)}
+        elif projid == "42003":
+            proj_params = {
+                "basemap": {"projection": "ortho", "lon_0": lon0, "lat_0": lat0},
+                "bbox": "meter({},{})".format(lon0, lat0)}
+        elif projid == "42004":
+            proj_params = {
+                "basemap": {"projection": "cyl"},
+                "bbox": "meter({},{})".format(lon0, lat0)}
+        elif projid == "42005":
+            proj_params = {
+                "basemap": {"projection": "moll", "lon_0": lon0, "lat_0": lat0},
+                "bbox": "meter???"}
+        else:
+            raise ValueError("unspecified AUTO2 code: '%s'", proj)
+
+    elif proj.startswith("epsg:"):
+        epsg = proj[5:]
+        if epsg.startswith("777") and len(epsg) == 8:  # user defined MSS code. deprecated.
+            logging.warning("Using deprecated MSS-specific EPSG code. Switch to 'MSS:stere' instead.")
+            lat_0, lon_0 = int(epsg[3:5]), int(epsg[5:])
+            proj_params = {
+                "basemap": {"projection": "stere", "lat_0": lat_0, "lon_0": lon_0},
+                "bbox": "degree"}
+        elif epsg.startswith("778") and len(epsg) == 8:  # user defined MSS code. deprecated.
+            logging.warning("Using deprecated MSS-specific EPSG code. Switch to 'MSS:stere' instead.")
+            lat_0, lon_0 = int(epsg[3:5]), int(epsg[5:])
+            proj_params = {
+                "basemap": {"projection": "stere", "lat_0": -lat_0, "lon_0": lon_0},
+                "bbox": "degree"}
+        elif epsg in ("4258", "4326"):
+            proj_params = {"basemap": {"epsg": epsg}, "bbox": "degree"}
+        elif epsg in ("3031", "3412"):
+            proj_params = {"basemap": {"epsg": epsg}, "bbox": "meter(0,-90)"}
+        elif epsg in ("3411", "3413", "3575", "3995"):
+            proj_params = {"basemap": {"epsg": epsg}, "bbox": "meter(0,90)"}
+        elif epsg in ("3395", "3857"):
+            proj_params = {"basemap": {"epsg": epsg}, "bbox": "meter(0,0)"}
+        elif epsg in ("4839"):
+            proj_params = {"basemap": {"epsg": epsg}, "bbox": "meter(10.5,51)"}
+        elif epsg in ("31467"):
+            proj_params = {"basemap": {"epsg": epsg}, "bbox": "meter(-20.9631343,0.0037502)"}
+        elif epsg in ("31468"):
+            proj_params = {"basemap": {"epsg": epsg}, "bbox": "meter(-25.4097892,0.0037466)"}
+        else:
+            raise ValueError("EPSG code not supported by basemap module: '%s'", proj)
+
+    elif proj.startswith("mss:"):
+        # some MSS-specific codes
+        params = proj[4:].split(",")
+        name = params[0]
+        if name == "stere":
+            lon0, lat0, lat_ts = params[1:]
+            proj_params = {
+                "basemap": {"projection": name, "lat_0": lat0, "lon_0": lon0, "lat_ts": lat_ts},
+                "bbox": "degree"}
+        elif name == "cass":
+            lon0, lat0 = params[1:]
+            proj_params = {
+                "basemap": {"projection": name, "lon_0": lon0, "lat_0": lat0},
+                "bbox": "degree"}
+        elif name == "lcc":
+            lon0, lat0, lat1, lat2 = params[1:]
+            proj_params = {
+                "basemap": {"projection": name, "lon_0": lon0, "lat_0": lat0, "lat_1": lat1, "lat_2": lat2},
+                "bbox": "degree"}
+        elif name == "merc":
+            lat_ts = params[1]
+            proj_params = {
+                "basemap": {"projection": name, "lat_ts": lat_ts},
+                "bbox": "degree"}
+        else:
+            raise ValueError("unknown MSS projection: '%s'", proj)
+
+    else:
+        raise ValueError("unknown projection: '%s'", proj)
+    logging.debug("Identified CRS '%s' as '%s'", proj, proj_params)
     return proj_params
 
 
@@ -269,7 +371,7 @@ def interpolate_vertsec(data3D, data3D_lats, data3D_lons, lats, lons):
     data3D can be on an IRREGULAR lat/lon grid, coordinates given by lats, lons.
     The lats, lons arrays can have arbitrary order, they do not have to be uniform.
     """
-    # Create an empty field to accomodate the curtain.
+    # Create an empty field to accommodate the curtain.
     curtain = np.zeros([data3D.shape[0], len(lats)])
 
     # Transform lat/lon values to array index space. This is necessary to use
@@ -397,9 +499,26 @@ def path_points(points, numpoints=100, connection='linear'):
     return [np.asarray(_x) for _x in (lats, lons, times)]
 
 
-class CaseInsensitiveMultiDict(paste.util.multidict.MultiDict):
-    """Extension to paste.util.multidict.MultiDict that makes the MultiDict
-       case-insensitive.
+def convert_pressure_to_vertical_axis_measure(vertical_axis, pressure):
+    """
+    vertical_axis can take following values
+    - pressure altitude
+    - flight level
+    - pressure
+    """
+    if vertical_axis == "pressure":
+        return float(pressure / 100)
+    elif vertical_axis == "flight level":
+        return pressure2flightlevel(pressure)
+    elif vertical_axis == "pressure altitude":
+        return pressure2flightlevel(pressure) / 32.8
+    else:
+        return pressure
+
+
+class CaseInsensitiveMultiDict(werkzeug.datastructures.ImmutableMultiDict):
+    """Extension to werkzeug.datastructures.ImmutableMultiDict
+    that makes the MultiDict case-insensitive.
 
     The only overridden method is __getitem__(), which converts string keys
     to lower case before carrying out comparisons.
@@ -411,7 +530,7 @@ class CaseInsensitiveMultiDict(paste.util.multidict.MultiDict):
     def __getitem__(self, key):
         if hasattr(key, 'lower'):
             key = key.lower()
-        for k, v in self._items:
+        for k, v in self.items():
             if hasattr(k, 'lower'):
                 k = k.lower()
             if k == key:
